@@ -1,7 +1,7 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2003, 2006 Andrea Paternesi
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2008 Janek Kozicki
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "Location.h"
 #include "shieldset.h"
 #include "cityset.h"
+#include "stackreflist.h"
 
 class MapGenerator;
 class XML_Helper;
@@ -43,6 +44,7 @@ class City;
 class Temple;
 class Bridge;
 class Ruin;
+class Armyset;
 
 /** Class representing the map in the game
   * 
@@ -124,13 +126,38 @@ class GameMap: public sigc::trackable
         void setTile(Vector<int> p, Maptile *t) {return setTile(p.x, p.y, t);}
 
 	static City* getCity(Vector<int> pos);
+        static City* getCity(Movable *m) {return getCity(m->getPos());}
+
+	static City* getEnemyCity(Vector<int> pos);
 	static Ruin* getRuin(Vector<int> pos);
+        static Ruin* getRuin(Movable *m) {return getRuin(m->getPos());}
 	static Temple* getTemple(Vector<int> pos);
+        static Temple* getTemple(Movable *m) {return getTemple(m->getPos());}
 	static Port* getPort(Vector<int> pos);
 	static Road* getRoad(Vector<int> pos);
 	static Bridge* getBridge(Vector<int> pos);
 	static Signpost* getSignpost(Vector<int> pos);
+        static Signpost* getSignpost(Movable *m) {return getSignpost(m->getPos());}
 	static Stack* getStack(Vector<int> pos);
+	static StackTile* getStacks(Vector<int> pos);
+	static Stack *groupStacks(Vector<int> pos);
+	static void groupStacks(Stack *s);
+	static bool canJoin(const Stack *src, Stack *dest);
+	static bool canJoin(const Stack *stack, Vector<int> pos);
+	static bool canAddArmy(Vector<int> pos);
+	static bool canAddArmies(Vector<int> dest, guint32 stackSize);
+	static Stack* getFriendlyStack(Vector<int> pos);
+        //static StackReflist getFriendlyStacks(Vector<int> pos, Player *player = NULL);
+	static std::list<Stack*> getFriendlyStacks(Vector<int> pos, Player *player = NULL);
+	static Stack* getEnemyStack(Vector<int> pos);
+	static std::list<Stack*> getEnemyStacks(std::list<Vector<int> > posns);
+	static std::list<Stack*> getEnemyStacks(Vector<int> pos, Player *player = NULL);
+	static std::list<Stack*> getNearbyFriendlyStacks(Vector<int> pos, int dist);
+	static std::list<Stack*> getNearbyEnemyStacks(Vector<int> pos, int dist);
+
+        static std::list<Vector<int> > getNearbyPoints(Vector<int> pos, int dist);
+	static guint32 countArmyUnits(Vector<int> pos);
+	static MapBackpack *getBackpack(Vector<int> pos);
 
         //! Get the tile object at position (x,y)
         Maptile* getTile(int x, int y) const;
@@ -141,10 +168,14 @@ class GameMap: public sigc::trackable
 	//! Add an army to this location on the map
 	Stack* addArmy(Location *l, Army *a);
         Stack* addArmy(Vector<int> pos, Army *a);
+        Stack* addArmyAtPos(Vector<int> pos, Army *a);
 
 	//! Go find a player's planted standard on the map
         Vector<int> findPlantedStandard(Player *p);
 
+	//! go find the player's stack, the slow way.
+	Vector<int> findStack(guint32 id);
+
         /** Fill the map using the data supplied by a map generator
           * 
           * @param generator        the generator which supplies the data
@@ -169,12 +200,14 @@ class GameMap: public sigc::trackable
 	//! figure out where a non-flying unit can't go
         void calculateBlockedAvenues();
 	void calculateBlockedAvenue(int i, int j);
+	void updateStackPositions();
 
 	/** Smooth a portion of the map.
 	 *
 	 * Give each tile in the prescribed area the preferred picture for 
 	 * the underlying terrain tile.
 	 */
+	void applyTileStyles (Rectangle r, bool smooth_terrain);
 	void applyTileStyles (int minx, int miny, int maxx, int maxy,
 			      bool smooth_terrain);
 	void applyTileStyle (int i, int j);
@@ -188,6 +221,53 @@ class GameMap: public sigc::trackable
 	Vector<int> findNearestObjectToTheEast(Vector<int> pos);
 	Vector<int> findNearestObjectToTheWest(Vector<int> pos);
 
+	void switchArmysets(Armyset *armyset);
+	void switchCityset(Cityset *cityset);
+	void switchShieldset(Shieldset *shieldset);
+	void switchTileset(Tileset *tileset);
+
+	bool moveBuilding(Vector<int> from, Vector<int> to);
+	bool canPutBuilding(Maptile::Building bldg, guint32 size, Vector<int> to, bool making_islands = true);
+	bool canPutStack(guint32 size, Player *p, Vector<int> to);
+	bool moveStack(Stack *stack, Vector<int> to);
+
+	void moveBackpack(Vector<int> from, Vector<int> to);
+	guint32 getBuildingSize(Vector<int> tile);
+	Maptile::Building getBuilding(Vector<int> tile);
+	Tile::Type getTerrainType(Vector<int> tile);
+	void setBuilding(Vector<int> tile, Maptile::Building building);
+
+	bool putCity(City *c);
+	bool removeCity(Vector<int> pos);
+	bool putRuin(Ruin *r);
+	bool removeRuin(Vector<int> pos);
+	bool putTemple(Temple *t);
+	bool removeTemple(Vector<int> pos);
+	bool putRoad(Road *r);
+	bool removeRoad(Vector<int> pos);
+	bool putBridge(Bridge *b);
+	bool removeBridge(Vector<int> pos);
+	bool putSignpost(Signpost *s);
+	bool removeSignpost(Vector<int> pos);
+	bool putPort(Port *p);
+	bool removePort(Vector<int> pos);
+	bool putStack(Stack *s);
+	void removeStack(Stack *s);
+
+	Location *getLocation(Vector<int> pos);
+
+	bool checkCityAccessibility();
+
+        static Vector<int> getCenterOfMap();
+
+        Rectangle putTerrain(Rectangle r, Tile::Type type, 
+                             int tile_style_id = -1, 
+                             bool always_alter_tilestyles = false);
+
+
+        static int calculateTilesPerOverviewMapTile(int width, int height);
+        static int calculateTilesPerOverviewMapTile();
+
     protected:
         //! Create the map with the given tileset
         GameMap(std::string TilesetName, std::string ShieldsetName,
@@ -201,7 +281,6 @@ class GameMap: public sigc::trackable
     private:
         //! Callback for item loading used during loading.
         bool loadItems(std::string tag, XML_Helper* helper);
-        Stack* addArmyAtPos(Vector<int> pos, Army *a);
         bool isBlockedAvenue(int x, int y, int destx, int desty);
         bool isDock(Vector<int> pos);
 	void close_circles (int minx, int miny, int maxx, int maxy);
@@ -216,6 +295,15 @@ class GameMap: public sigc::trackable
 						   int j);
 	bool are_those_tiles_similar(Tile::Type outer_tile,Tile::Type inner_tile, bool checking_loneliness);
 	Vector<int> findNearestObjectInDir(Vector<int> pos, Vector<int> dir);
+	void putBuilding(LocationBox *b, Maptile::Building building);
+	void removeBuilding(LocationBox *b);
+
+	void updateShips(Vector<int> pos);
+
+
+	static std::list<Stack*> getNearbyStacks(Vector<int> pos, int dist, bool friendly);
+
+	static bool offmap(int x, int y);
 
         // Data
         static GameMap* s_instance;
