@@ -23,7 +23,6 @@
 #include "rectangle.h"
 
 #include "GraphicsCache.h"
-#include "GraphicsLoader.h"
 #include "armysetlist.h"
 #include "shieldsetlist.h"
 #include "tilesetlist.h"
@@ -36,9 +35,9 @@
 #include "GameMap.h"
 #include "city.h"
 #include "stack.h"
-#include "rgb_shift.h"
 #include "gui/image-helpers.h"
 #include "FogMap.h"
+#include "hero.h"
 
 //#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl<<std::flush;}
 #define debug(x)
@@ -66,6 +65,7 @@ bool operator <(ArmyCacheItem lhs, ArmyCacheItem rhs)
 struct ShipCacheItem
 {
     guint32 player_id;
+    guint32 armyset;
     PixMask* surface;
 };
 
@@ -73,6 +73,7 @@ struct ShipCacheItem
 struct PlantedStandardCacheItem
 {
     guint32 player_id;
+    guint32 armyset;
     PixMask* surface;
 };
 
@@ -80,6 +81,7 @@ struct PlantedStandardCacheItem
 struct NewLevelCacheItem
 {
     guint32 player_id;
+    guint32 gender;
     PixMask* surface;
 };
 
@@ -225,6 +227,31 @@ struct TileCacheItem
   guint32 cityset;
   PixMask* surface;
 };
+
+struct PortCacheItem
+{
+  guint32 cityset;
+  PixMask *surface;
+};
+
+struct SignpostCacheItem
+{
+  guint32 cityset;
+  PixMask *surface;
+};
+
+struct BagCacheItem
+{
+  guint32 armyset;
+  PixMask *surface;
+};
+
+struct ExplosionCacheItem
+{
+  guint32 tileset;
+  PixMask *surface;
+};
+
 bool operator <(TileCacheItem lhs, TileCacheItem rhs)
 {
   return memcmp(&lhs, &rhs, sizeof (TileCacheItem) - sizeof (PixMask*)) < 0;
@@ -261,15 +288,14 @@ GraphicsCache::GraphicsCache()
     loadMedalPics();
     loadNewLevelPics();
 
-    d_smallruinedcity = GraphicsLoader::getMiscPicture("smallruinedcity.png");
-    d_smallhero = GraphicsLoader::getMiscPicture("hero.png");
-    d_smallinactivehero = GraphicsLoader::getMiscPicture("hero-inactive.png");
-    d_small_ruin_unexplored = GraphicsLoader::getMiscPicture("smallunexploredruin.png");
+    d_smallruinedcity = getMiscPicture("smallruinedcity.png");
+    d_smallhero = getMiscPicture("hero.png");
+    d_smallinactivehero = getMiscPicture("hero-inactive.png");
+    d_small_ruin_unexplored = getMiscPicture("smallunexploredruin.png");
     d_small_stronghold_unexplored = 
-      GraphicsLoader::getMiscPicture("smallunexploredstronghold.png");
-    d_small_ruin_explored = GraphicsLoader::getMiscPicture("smallexploredruin.png");
-    d_small_temple = GraphicsLoader::getMiscPicture("smalltemple.png");
-    d_bag = GraphicsLoader::getMiscPicture("items.png");
+      getMiscPicture("smallunexploredstronghold.png");
+    d_small_ruin_explored = getMiscPicture("smallexploredruin.png");
+    d_small_temple = getMiscPicture("smalltemple.png");
 }
 
 GraphicsCache::~GraphicsCache()
@@ -293,9 +319,10 @@ GraphicsCache::~GraphicsCache()
     delete d_small_ruin_unexplored;
     delete d_small_stronghold_unexplored;
     delete d_small_ruin_explored;
-    delete d_bag;
-    delete d_newlevel;
-    delete d_newlevelmask;
+    delete d_newlevel_male;
+    delete d_newlevel_female;
+    delete d_newlevelmask_male;
+    delete d_newlevelmask_female;
 }
 
 PixMask* GraphicsCache::getSmallRuinedCityPic()
@@ -328,26 +355,6 @@ PixMask* GraphicsCache::getSmallTemplePic()
   return d_small_temple;
 }
 
-PixMask* GraphicsCache::getBagPic()
-{
-  return d_bag;
-}
-
-PixMask* GraphicsCache::getPortPic()
-{
-  return GameMap::getInstance()->getCityset()->getPortImage();
-}
-
-PixMask* GraphicsCache::getExplosionPic()
-{
-  return GameMap::getInstance()->getTileset()->getExplosionImage();
-}
-
-PixMask* GraphicsCache::getSignpostPic()
-{
-  return GameMap::getInstance()->getCityset()->getSignpostImage();
-}
-
 PixMask* GraphicsCache::getMoveBonusPic(guint32 bonus, bool has_ship)
 {
   guint32 type;
@@ -395,7 +402,7 @@ PixMask* GraphicsCache::getShipPic(const Player* p)
     ShipCacheItem* myitem;
     for (it = d_shiplist.begin(); it != d_shiplist.end(); it++)
     {
-        if ((*it)->player_id == p->getId())
+        if ((*it)->player_id == p->getId() && (*it)->armyset == p->getArmyset())
         {
             myitem = (*it);
             
@@ -413,14 +420,14 @@ PixMask* GraphicsCache::getShipPic(const Player* p)
     return myitem->surface;
 }
 
-PixMask* GraphicsCache::getNewLevelPic(const Player* p)
+PixMask* GraphicsCache::getNewLevelPic(const Player* p, guint32 gender)
 {
     debug("getting new level pic " <<p->getName())
     std::list<NewLevelCacheItem*>::iterator it;
     NewLevelCacheItem* myitem;
     for (it = d_newlevellist.begin(); it != d_newlevellist.end(); it++)
     {
-        if ((*it)->player_id == p->getId())
+        if ((*it)->player_id == p->getId() && (*it)->gender == gender)
         {
             myitem = (*it);
             
@@ -434,7 +441,132 @@ PixMask* GraphicsCache::getNewLevelPic(const Player* p)
     // We are still here, so the graphic is not in the cache. 
     // addNewLevelPic calls checkPictures on its own, so we can 
     // simply return the surface
-    myitem = addNewLevelPic(p);
+    myitem = addNewLevelPic(p, gender);
+
+    return myitem->surface;
+}
+
+PixMask* GraphicsCache::getPortPic()
+{
+  return getPortPic(GameMap::getInstance()->getCityset()->getId());
+}
+
+PixMask* GraphicsCache::getPortPic(guint32 cityset)
+{
+    debug("getting port pic " << cityset)
+    std::list<PortCacheItem*>::iterator it;
+    PortCacheItem* myitem;
+    for (it = d_portlist.begin(); it != d_portlist.end(); it++)
+    {
+        if ((*it)->cityset == cityset)
+        {
+            myitem = (*it);
+            
+            // put the item on the last place (==last touched)
+            d_portlist.erase(it);
+            d_portlist.push_back(myitem);
+            
+            return myitem->surface;
+        }
+    }
+    // We are still here, so the graphic is not in the cache. 
+    // addPortPic calls checkPictures on its own, so we can 
+    // simply return the surface
+    myitem = addPortPic(cityset);
+
+    return myitem->surface;
+}
+
+PixMask* GraphicsCache::getSignpostPic()
+{
+  return getSignpostPic(GameMap::getInstance()->getCityset()->getId());
+}
+
+PixMask* GraphicsCache::getSignpostPic(guint32 cityset)
+{
+    debug("getting signpost pic " << cityset)
+    std::list<SignpostCacheItem*>::iterator it;
+    SignpostCacheItem* myitem;
+    for (it = d_signpostlist.begin(); it != d_signpostlist.end(); it++)
+    {
+        if ((*it)->cityset == cityset)
+        {
+            myitem = (*it);
+            
+            // put the item on the last place (==last touched)
+            d_signpostlist.erase(it);
+            d_signpostlist.push_back(myitem);
+            
+            return myitem->surface;
+        }
+    }
+    // We are still here, so the graphic is not in the cache. 
+    // addSignpostPic calls checkPictures on its own, so we can 
+    // simply return the surface
+    myitem = addSignpostPic(cityset);
+
+    return myitem->surface;
+}
+
+PixMask* GraphicsCache::getBagPic()
+{
+  guint32 armyset = Playerlist::getActiveplayer()->getArmyset();
+  return getBagPic(armyset);
+}
+
+PixMask* GraphicsCache::getBagPic(guint32 armyset)
+{
+    debug("getting bag pic " << armyset)
+    std::list<BagCacheItem*>::iterator it;
+    BagCacheItem* myitem;
+    for (it = d_baglist.begin(); it != d_baglist.end(); it++)
+    {
+        if ((*it)->armyset == armyset)
+        {
+            myitem = (*it);
+            
+            // put the item on the last place (==last touched)
+            d_baglist.erase(it);
+            d_baglist.push_back(myitem);
+            
+            return myitem->surface;
+        }
+    }
+    // We are still here, so the graphic is not in the cache. 
+    // addBagPic calls checkPictures on its own, so we can 
+    // simply return the surface
+    myitem = addBagPic(armyset);
+
+    return myitem->surface;
+}
+
+PixMask* GraphicsCache::getExplosionPic()
+{
+  return getExplosionPic(GameMap::getInstance()->getTileset()->getId());
+}
+
+PixMask* GraphicsCache::getExplosionPic(guint32 tileset)
+{
+    debug("getting explosion pic " << tileset)
+    std::list<ExplosionCacheItem*>::iterator it;
+    ExplosionCacheItem* myitem;
+    for (it = d_explosionlist.begin(); it != d_explosionlist.end(); it++)
+    {
+        if ((*it)->tileset == tileset)
+        {
+            myitem = (*it);
+            
+            // put the item on the last place (==last touched)
+            d_explosionlist.erase(it);
+            d_explosionlist.push_back(myitem);
+            
+            return myitem->surface;
+        }
+    }
+    // We are still here, so the graphic is not in the cache. 
+    // addExplosionPic calls checkPictures on its own, so we can 
+    // simply return the surface
+    myitem = addExplosionPic(tileset);
 
     return myitem->surface;
 }
@@ -446,7 +578,7 @@ PixMask* GraphicsCache::getPlantedStandardPic(const Player* p)
     PlantedStandardCacheItem* myitem;
     for (it = d_plantedstandardlist.begin(); it != d_plantedstandardlist.end(); it++)
     {
-        if ((*it)->player_id == p->getId())
+        if ((*it)->player_id == p->getId() && (*it)->armyset == p->getArmyset())
         {
             myitem = (*it);
             
@@ -911,6 +1043,8 @@ PixMask* GraphicsCache::getTowerPic(const Player* p, guint32 cityset)
 
 PixMask* GraphicsCache::getFlagPic(guint32 stack_size, const Player *p, guint32 tileset)
 {
+  if (stack_size > MAX_STACK_SIZE || p == NULL || tileset == 0)
+    return NULL;
     debug("GraphicsCache::getFlagPic " <<stack_size <<", player" <<p->getId())
 
     std::list<FlagCacheItem*>::iterator it;
@@ -1024,7 +1158,7 @@ PixMask* GraphicsCache::getProdShieldPic(guint32 type, bool prod)
 }
 
 
-PixMask* GraphicsCache::applyMask(PixMask* image, PixMask* mask, struct rgb_shift shifts, bool isNeutral)
+PixMask* GraphicsCache::applyMask(PixMask* image, PixMask* mask, Gdk::Color colour, bool isNeutral)
 {
   int width = image->get_width();
   int height = image->get_height();
@@ -1049,9 +1183,9 @@ PixMask* GraphicsCache::applyMask(PixMask* image, PixMask* mask, struct rgb_shif
 
 	if (copy[base+3] != 0)
 	  {
-	    copy[base+0] >>= (shifts.r);
-	    copy[base+1] >>= (shifts.g);
-	    copy[base+2] >>= (shifts.b);
+	    copy[base+0] = colour.get_red_p() *copy[base+0];
+	    copy[base+1] = colour.get_green_p() * copy[base+1];
+	    copy[base+2] = colour.get_blue_p() * copy[base+2];
 	  }
       }
   Glib::RefPtr<Gdk::Pixbuf> colouredmask = 
@@ -1107,7 +1241,7 @@ PixMask* GraphicsCache::greyOut(PixMask* image)
 
 PixMask* GraphicsCache::applyMask(PixMask* image, PixMask* mask, const Player* p)
 {
-  return applyMask(image, mask, p->getMaskColorShifts(),
+  return applyMask(image, mask, p->getColor(),
 		   Playerlist::getInstance()->getNeutral()->getId() == p->getId());
 }
 
@@ -1203,6 +1337,34 @@ void GraphicsCache::checkPictures()
   if (d_cachesize < maxcache)
     return;
 
+  // next, kill port pics
+  while (d_portlist.size() > 1)
+    eraseLastPortItem();
+
+  if (d_cachesize < maxcache)
+    return;
+
+  // next, kill signpost pics
+  while (d_signpostlist.size() > 1)
+    eraseLastSignpostItem();
+
+  if (d_cachesize < maxcache)
+    return;
+
+  // next, kill bag pics
+  while (d_baglist.size() > 1)
+    eraseLastBagItem();
+
+  if (d_cachesize < maxcache)
+    return;
+
+  // next, kill explosion pics
+  while (d_explosionlist.size() > 1)
+    eraseLastExplosionItem();
+
+  if (d_cachesize < maxcache)
+    return;
+
   // next, kill movement bonus pics
   while (d_movebonuslist.size() > 20)
     eraseLastMoveBonusItem();
@@ -1238,11 +1400,11 @@ void GraphicsCache::drawTilePic(PixMask *surface, int fog_type_id, bool has_bag,
     case Maptile::TEMPLE:
       getTemplePic(building_subtype, cityset)->blit(building_tile, ts, pixmap); break;
     case Maptile::SIGNPOST:
-      getSignpostPic()->blit(building_tile, ts, pixmap); break;
+      getSignpostPic(cityset)->blit(building_tile, ts, pixmap); break;
     case Maptile::ROAD:
       getRoadPic(building_subtype)->blit(building_tile, ts, pixmap); break;
     case Maptile::PORT:
-      getPortPic()->blit(building_tile, ts, pixmap); break;
+      getPortPic(cityset)->blit(building_tile, ts, pixmap); break;
     case Maptile::BRIDGE:
       getBridgePic(building_subtype)->blit(building_tile, ts, pixmap); break;
       break;
@@ -1257,7 +1419,11 @@ void GraphicsCache::drawTilePic(PixMask *surface, int fog_type_id, bool has_bag,
     }
 
   if (has_bag)
-    getBagPic()->blit(pixmap, Vector<int>(ts - 18, ts - 18));
+    {
+      PixMask *pic = getBagPic();
+      Vector<int>bagsize = Vector<int>(pic->get_width(), pic->get_height());
+      pic->blit(pixmap, Vector<int>(ts,ts)-bagsize);
+    }
 
   if (stack_player_id > -1)
     {
@@ -1436,6 +1602,7 @@ ShipCacheItem* GraphicsCache::addShipPic(const Player* p)
 
   ShipCacheItem* myitem = new ShipCacheItem();
   myitem->player_id = p->getId();
+  myitem->armyset = p->getArmyset();
 
   Armysetlist *al = Armysetlist::getInstance();
   PixMask*ship = al->getShipPic(p->getArmyset());
@@ -1458,15 +1625,19 @@ ShipCacheItem* GraphicsCache::addShipPic(const Player* p)
   return myitem;
 }
 
-NewLevelCacheItem* GraphicsCache::addNewLevelPic(const Player* p)
+NewLevelCacheItem* GraphicsCache::addNewLevelPic(const Player* p, guint32 gender)
 {
   debug("ADD new level pic: " <<p->getName())
 
   NewLevelCacheItem* myitem = new NewLevelCacheItem();
   myitem->player_id = p->getId();
+  myitem->gender = gender;
 
   // copy the pixmap including player colors
-  myitem->surface = applyMask(d_newlevel, d_newlevelmask, p);
+  if (gender == Hero::FEMALE)
+    myitem->surface = applyMask(d_newlevel_female, d_newlevelmask_female, p);
+  else
+    myitem->surface = applyMask(d_newlevel_male, d_newlevelmask_male, p);
 
   //now the final preparation steps:
   //a) add the size
@@ -1483,12 +1654,120 @@ NewLevelCacheItem* GraphicsCache::addNewLevelPic(const Player* p)
   return myitem;
 }
 
+PortCacheItem* GraphicsCache::addPortPic(guint32 cityset)
+{
+  debug("ADD port pic: " << cityset);
+
+  PortCacheItem* myitem = new PortCacheItem();
+  myitem->cityset = cityset;
+
+  Citysetlist *csl = Citysetlist::getInstance();
+  Cityset *cs = csl->getCityset(cityset);
+
+  // copy the pixmap
+  myitem->surface = cs->getPortImage()->copy();
+
+  //now the final preparation steps:
+  //a) add the size
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize += myitem->surface->get_depth()/8 * size;
+
+  //b) add the entry to the list
+  d_portlist.push_back(myitem);
+
+  //c) check if the cache size is too large
+  checkPictures();
+
+  //we are finished, so return the pic
+  return myitem;
+}
+
+ExplosionCacheItem* GraphicsCache::addExplosionPic(guint32 tileset)
+{
+  debug("ADD explosion pic: " << tileset);
+
+  ExplosionCacheItem* myitem = new ExplosionCacheItem();
+  myitem->tileset = tileset;
+
+  // copy the pixmap
+  myitem->surface = 
+    Tilesetlist::getInstance()->getTileset(tileset)->getExplosionImage()->copy();
+
+  //now the final preparation steps:
+  //a) add the size
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize += myitem->surface->get_depth()/8 * size;
+
+  //b) add the entry to the list
+  d_explosionlist.push_back(myitem);
+
+  //c) check if the cache size is too large
+  checkPictures();
+
+  //we are finished, so return the pic
+  return myitem;
+}
+
+BagCacheItem* GraphicsCache::addBagPic(guint32 armyset)
+{
+  debug("ADD bad pic: " << armyset);
+
+  BagCacheItem* myitem = new BagCacheItem();
+  myitem->armyset = armyset;
+
+  // copy the pixmap
+  myitem->surface = Armysetlist::getInstance()->getBagPic(armyset)->copy();
+
+  //now the final preparation steps:
+  //a) add the size
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize += myitem->surface->get_depth()/8 * size;
+
+  //b) add the entry to the list
+  d_baglist.push_back(myitem);
+
+  //c) check if the cache size is too large
+  checkPictures();
+
+  //we are finished, so return the pic
+  return myitem;
+}
+
+SignpostCacheItem* GraphicsCache::addSignpostPic(guint32 cityset)
+{
+  debug("ADD signpost pic: " << cityset);
+
+  SignpostCacheItem* myitem = new SignpostCacheItem();
+  myitem->cityset = cityset;
+
+  Citysetlist *csl = Citysetlist::getInstance();
+  Cityset *cs = csl->getCityset(cityset);
+
+  // copy the pixmap
+  myitem->surface = cs->getSignpostImage()->copy();
+
+  //now the final preparation steps:
+  //a) add the size
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize += myitem->surface->get_depth()/8 * size;
+
+  //b) add the entry to the list
+  d_signpostlist.push_back(myitem);
+
+  //c) check if the cache size is too large
+  checkPictures();
+
+  //we are finished, so return the pic
+  return myitem;
+}
+
 PlantedStandardCacheItem* GraphicsCache::addPlantedStandardPic(const Player* p)
 {
   debug("ADD planted standard pic: " <<p->getName())
 
     PlantedStandardCacheItem* myitem = new PlantedStandardCacheItem();
   myitem->player_id = p->getId();
+  myitem->armyset = p->getArmyset();
 
   Armysetlist *al = Armysetlist::getInstance();
   PixMask*standard = al->getStandardPic(p->getArmyset());
@@ -1544,6 +1823,7 @@ RuinCacheItem* GraphicsCache::addRuinPic(int type, guint32 cityset)
   //now create the cache item and add the size
   RuinCacheItem* myitem = new RuinCacheItem();
   myitem->type = type;
+  myitem->cityset = cityset;
   myitem->surface = mysurf;
   myitem->cityset = cityset;
 
@@ -1613,6 +1893,7 @@ FogCacheItem* GraphicsCache::addFogPic(FogCacheItem *item)
   //now create the cache item and add the size
   FogCacheItem* myitem = new FogCacheItem();
   *myitem = *item;
+  myitem->surface = mysurf;
 
   d_foglist.push_back(myitem);
   std::list<FogCacheItem*>::iterator it = d_foglist.end();
@@ -1905,6 +2186,18 @@ void GraphicsCache::clear()
   while (!d_plantedstandardlist.empty())
     eraseLastPlantedStandardItem();
 
+  while (!d_portlist.empty())
+    eraseLastPortItem();
+
+  while (!d_signpostlist.empty())
+    eraseLastSignpostItem();
+
+  while (!d_baglist.empty())
+    eraseLastBagItem();
+
+  while (!d_explosionlist.empty())
+    eraseLastExplosionItem();
+
   while (!d_flaglist.empty())
     eraseLastFlagItem();
 
@@ -2133,6 +2426,66 @@ void GraphicsCache::eraseLastNewLevelItem()
   delete myitem;
 }
 
+void GraphicsCache::eraseLastExplosionItem()
+{
+  if (d_explosionlist.empty())
+    return;
+
+  ExplosionCacheItem* myitem = *(d_explosionlist.begin());
+  d_explosionlist.erase(d_explosionlist.begin());
+
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize -= myitem->surface->get_depth()/8 * size;
+
+  delete myitem->surface;
+  delete myitem;
+}
+
+void GraphicsCache::eraseLastBagItem()
+{
+  if (d_baglist.empty())
+    return;
+
+  BagCacheItem* myitem = *(d_baglist.begin());
+  d_baglist.erase(d_baglist.begin());
+
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize -= myitem->surface->get_depth()/8 * size;
+
+  delete myitem->surface;
+  delete myitem;
+}
+
+void GraphicsCache::eraseLastSignpostItem()
+{
+  if (d_signpostlist.empty())
+    return;
+
+  SignpostCacheItem* myitem = *(d_signpostlist.begin());
+  d_signpostlist.erase(d_signpostlist.begin());
+
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize -= myitem->surface->get_depth()/8 * size;
+
+  delete myitem->surface;
+  delete myitem;
+}
+
+void GraphicsCache::eraseLastPortItem()
+{
+  if (d_portlist.empty())
+    return;
+
+  PortCacheItem* myitem = *(d_portlist.begin());
+  d_portlist.erase(d_portlist.begin());
+
+  int size = myitem->surface->get_width() * myitem->surface->get_height();
+  d_cachesize -= myitem->surface->get_depth()/8 * size;
+
+  delete myitem->surface;
+  delete myitem;
+}
+
 void GraphicsCache::eraseLastPlantedStandardItem()
 {
   if (d_plantedstandardlist.empty())
@@ -2324,9 +2677,14 @@ void GraphicsCache::loadMedalPics()
 void GraphicsCache::loadNewLevelPics()
 {
   std::vector<PixMask* > half;
-  half = disassemble_row(File::getMiscFile("various/hero-newlevel.png"), 2);
-  d_newlevel = half[0];
-  d_newlevelmask = half[1];
+  half = disassemble_row(File::getMiscFile("various/hero-newlevel-male.png"), 
+			 2);
+  d_newlevel_male = half[0];
+  d_newlevelmask_male = half[1];
+  half = disassemble_row(File::getMiscFile("various/hero-newlevel-female.png"), 
+			 2);
+  d_newlevel_female = half[0];
+  d_newlevelmask_female = half[1];
 }
 
 void GraphicsCache::loadMoveBonusPics()
@@ -2347,15 +2705,15 @@ void GraphicsCache::loadMoveBonusPics()
 
 bool GraphicsCache::loadFlagImages(std::string filename, guint32 size, std::vector<PixMask* > &images, std::vector<PixMask* > &masks)
 {
-  images = disassemble_row(filename, MAX_STACK_SIZE, true);
-  for (unsigned int i = 0; i < MAX_STACK_SIZE; i++)
+  images = disassemble_row(filename, FLAG_TYPES, true);
+  for (unsigned int i = 0; i < FLAG_TYPES; i++)
     {
       if (images[i]->get_width() != (int)size)
 	PixMask::scale(images[i], size, size);
 
     }
-  masks = disassemble_row(filename, MAX_STACK_SIZE, false);
-  for (unsigned int i = 0; i < MAX_STACK_SIZE; i++)
+  masks = disassemble_row(filename, FLAG_TYPES, false);
+  for (unsigned int i = 0; i < FLAG_TYPES; i++)
     {
       if (masks[i]->get_width() !=(int) size)
 	PixMask::scale(masks[i], size, size);
@@ -2370,3 +2728,14 @@ PixMask* GraphicsCache::getMedalPic(bool large, int type)
   else
     return d_medalpic[0][type];
 }
+
+PixMask* GraphicsCache::loadImage(std::string filename, bool alpha)
+{
+  return PixMask::create(filename);
+}
+
+PixMask* GraphicsCache::getMiscPicture(std::string picname, bool alpha)
+{
+  return loadImage(File::getMiscFile("/various/" + picname), alpha);
+}
+
