@@ -28,7 +28,6 @@
 
 #include "player.h"
 #include "defs.h"
-#include "rgb_shift.h"
 #include "PixMask.h"
 #include "maptile.h"
 
@@ -54,6 +53,10 @@ struct FogCacheItem;
 struct PlantedStandardCacheItem;
 struct NewLevelCacheItem;
 struct TileCacheItem;
+struct PortCacheItem;
+struct SignpostCacheItem;
+struct BagCacheItem;
+struct ExplosionCacheItem;
 class City;
 
 /** Soliton class for caching army and map images
@@ -102,7 +105,7 @@ class GraphicsCache
       HEART,
       GOTO_ARROW
     };
-        //! Function for getting/creating the soliton instance.
+        //! Method for getting/creating the soliton instance.
         static GraphicsCache* getInstance();
 
         //! Explicitely deletes the soliton instance
@@ -114,10 +117,10 @@ class GraphicsCache
         //! Get the current cache size, the maximum is in Configuration::s_cacheSize
         guint32 getCacheSize() const {return d_cachesize;}
 
-        /** Function for getting the army picture from the cache
+        /** Method for getting the army picture from the cache
           * 
-          * This function returns either the cached image of the given type or
-          * creates a new one and caches it. Use this function to access
+          * This method returns either the cached image of the given type or
+          * creates a new one and caches it. Use this method to access
           * army images! And: Don't touch the returned surface!! For performance
           * reasons you get the original surface which is also in the cache.
           *
@@ -137,10 +140,10 @@ class GraphicsCache
 	PixMask* getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid, guint32 tileset, guint32 cityset);
 	PixMask* getTilePic(int tile_style_id, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 tilesize, bool has_grid);
 
-        /** Function for getting the shield picture from the cache
+        /** Method for getting the shield picture from the cache
           * 
-          * This function returns either the cached image of the given type or
-          * creates a new one and caches it. Use this function to access
+          * This method returns either the cached image of the given type or
+          * creates a new one and caches it. Use this method to access
           * army images! And: Don't touch the returned surface!! For performance
           * reasons you get the original surface which is also in the cache.
           *
@@ -155,7 +158,7 @@ class GraphicsCache
         PixMask* getShieldPic(guint32 shieldset, guint32 type, guint32 colour);
         PixMask* getShieldPic(guint32 type, Player *p);
 
-        /** Function for getting a ruin picture
+        /** Method for getting a ruin picture
           *
           * @param type         the type of the ruin
           * @return image of the ruin 
@@ -163,7 +166,7 @@ class GraphicsCache
         PixMask* getRuinPic(int type);
         PixMask* getRuinPic(int type, guint32 cityset);
 
-        /** Function for getting a diplomacy icon
+        /** Method for getting a diplomacy icon
           *
           * @param type         o = small, or 1 = large.
           * @param state        the diplomatic state.  e.g. peace, war, etc
@@ -171,7 +174,7 @@ class GraphicsCache
           */
         PixMask* getDiplomacyPic(int type, Player::DiplomaticState state);
 
-        /** Function for getting a temple picture
+        /** Method for getting a temple picture
           *
           * @param type         the type of the temple
           * @return image of the temple
@@ -179,7 +182,7 @@ class GraphicsCache
         PixMask* getTemplePic(int type);
         PixMask* getTemplePic(int type, guint32 cityset);
 
-        /** Function for getting a road picture
+        /** Method for getting a road picture
           *
           * @param type         the type of the road
           * @return image of the road
@@ -187,7 +190,7 @@ class GraphicsCache
         PixMask* getRoadPic(int type, guint32 tileset);
         PixMask* getRoadPic(int type);
 
-        /** Function for getting a fog picture
+        /** Method for getting a fog picture
           *
           * @param type         the type of the fog
           * @return image of the fog
@@ -195,7 +198,7 @@ class GraphicsCache
         PixMask* getFogPic(int type, guint32 tileset);
         PixMask* getFogPic(int type);
 
-        /** Function for getting a bridge picture
+        /** Method for getting a bridge picture
           *
           * @param type         the type of the bridge 0=e/w 1=n/s
           * @return image of the bridge
@@ -203,14 +206,14 @@ class GraphicsCache
         PixMask* getBridgePic(int type, guint32 tileset);
         PixMask* getBridgePic(int type);
 
-        /** Function for getting a cursor picture
+        /** Method for getting a cursor picture
           *
           * @param type         the type of the cursor 
           * @return image of the cursor
           */
         PixMask* getCursorPic(int type);
 
-        /** Function for getting a ship picture.  This is the picture
+        /** Method for getting a ship picture.  This is the picture
 	  * that appears when the stack goes into the water.
           *
           * @param p            the player to colour the ship as
@@ -218,7 +221,7 @@ class GraphicsCache
           */
         PixMask* getShipPic(const Player* p);
 
-        /** Function for getting a standard picture.  This is the picture
+        /** Method for getting a standard picture.  This is the picture
 	  * that appears when the hero plants a flag..
           *
           * @param p            the player to colour the flag as
@@ -226,20 +229,52 @@ class GraphicsCache
           */
         PixMask* getPlantedStandardPic(const Player* p);
 
-	/** Function for getting a new-level picture.  This is the picture
+        /** Method for getting a port picture.  This is the picture
+	  * that appears often as an anchor on coastal regions.
+          *
+          * @return image of the port
+          */
+        PixMask* getPortPic();
+        PixMask* getPortPic(guint32 cityset);
+
+        /** Method for getting a signpost picture.  This is the picture
+	  * that appears as a little tiny sign on grassy tiles.
+          *
+          * @return image of the signpost
+          */
+        PixMask* getSignpostPic();
+        PixMask* getSignpostPic(guint32 cityset);
+
+        /** Method for getting a bag-of-items picture.  This is the picture
+	  * that shows when a hero drops one or more items on the ground.
+          *
+          * @return image of the sack of items
+          */
+        PixMask* getBagPic();
+        PixMask* getBagPic(guint32 armyset);
+
+        /** Method for getting an explosion picture.  This is the picture
+	  * that shows when stacks are fighting.
+          *
+          * @return image of the explosion.
+          */
+        PixMask* getExplosionPic();
+        PixMask* getExplosionPic(guint32 tileset);
+
+	/** Method for getting a new-level picture.  This is the picture
 	 * that appears when a hero gains a new level, and subsequently gets
 	 * to increase a stat.
 	 *
 	 * @param p the player to colour the image as.
 	 * @return new-level image.
 	 */
-        PixMask* getNewLevelPic(const Player* p);
+        PixMask* getNewLevelPic(const Player* p, guint32 gender);
 
-        /** Function for getting a city picture
+        /** Method for getting a city picture
           * 
           * For simplicity we have extended the basic_image/mask style to
           * cities as well, since it greatly reduces the number of images.
-          * Use this function solely to get city images, and don't touch the
+          * Use this method solely to get city images, and don't touch the
           * images!
           *
           * @param type         the level of the city; -1 returns the pic for
@@ -249,9 +284,9 @@ class GraphicsCache
           * @return image of the described city
           */
         PixMask* getCityPic(int type, const Player* p, guint32 cityset);
-        /** Another function for getting a city picture
+        /** Another method for getting a city picture
           *
-          * Most often, we don't need such a sophisticated function. So just
+          * Most often, we don't need such a sophisticated method. So just
           * supply the city instance and be happy. :)
           *
           * @param city     the city whose picture we want to get
@@ -260,9 +295,9 @@ class GraphicsCache
         PixMask* getCityPic(const City* city);
         PixMask* getCityPic(const City* city, guint32 cityset);
 
-        /** Function for getting tower pictures.
+        /** Method for getting tower pictures.
           *
-          * As with the other functions, use solely this function to get the tower 
+          * As with the other methods, use solely this method to get the tower 
           * images. And DON'T modify the images!
           *
           * @param p the player for which we want to get the tower
@@ -271,9 +306,9 @@ class GraphicsCache
         PixMask* getTowerPic(const Player *p);
 	PixMask* getTowerPic(const Player* p, guint32 cityset);
 
-        /** Function for getting flag pictures.
+        /** Method for getting flag pictures.
           *
-          * As with the other functions, use solely this function to get the flag
+          * As with the other methods, use solely this method to get the flag
           * images. And DON'T modify the images!
           *
           * @param stack    the stack for which we want to get the flag
@@ -284,9 +319,9 @@ class GraphicsCache
 	PixMask* getFlagPic(guint32 stack_size, const Player *p);
         PixMask* getFlagPic(guint32 stack_size, const Player *p, guint32 tileset);
 
-        /** Function for getting selector pictures.
+        /** Method for getting selector pictures.
           *
-          * As with the other functions, use solely this function to get the 
+          * As with the other methods, use solely this method to get the 
           * selector images. And DON'T modify the images!
           *
           * @param type the frame of the selector
@@ -297,9 +332,9 @@ class GraphicsCache
 
 	PixMask* getSelectorPic(guint32 type, guint32 frame, const Player *p);
 
-        /** Function for getting shield pictures.
+        /** Method for getting shield pictures.
           *
-          * As with the other functions, use solely this function to get the 
+          * As with the other methods, use solely this method to get the 
           * shield images. And DON'T modify the images!
           *
           * @param type small, medium or large shield size
@@ -312,19 +347,15 @@ class GraphicsCache
         PixMask* getSmallRuinedCityPic();
 	//! Return a small hero picture, either white (active==true) or black.
         PixMask* getSmallHeroPic(bool active);
-        PixMask* getBagPic();
-        PixMask* getPortPic();
-        PixMask* getExplosionPic();
-        PixMask* getSignpostPic();
         PixMask* getMoveBonusPic(guint32 bonus, bool has_ship);
         PixMask*getSmallTemplePic();
         PixMask*getSmallRuinExploredPic();
         PixMask* getSmallRuinUnexploredPic();
         PixMask* getSmallStrongholdUnexploredPic();
 
-        /** Function for getting production shield pictures.
+        /** Method for getting production shield pictures.
           *
-          * As with the other functions, use solely this function to get the 
+          * As with the other methods, use solely this method to get the 
           * shield images. And DON'T modify the images!
           *
           * @param type home/away/destination/source/invalid.  
@@ -355,15 +386,23 @@ class GraphicsCache
 
 	PixMask* applyMask(PixMask* image, PixMask* mask, const Player* p);
 
+	static PixMask* applyMask(PixMask* image, PixMask* mask, Gdk::Color colour, bool isNeutral);
 
-	static PixMask* applyMask(PixMask* image, PixMask* mask, struct rgb_shift shifts, bool isNeutral);
-        
 	static PixMask* greyOut(PixMask* image);
 
 	static bool loadSelectorImages(std::string filename, guint32 tilesize, std::vector<PixMask* > &images, std::vector<PixMask* > &masks);
 
 	static bool loadFlagImages(std::string filename, guint32 size, std::vector<PixMask* > &images, std::vector<PixMask* > &masks);
 
+        /** Load misc pic
+          * 
+          * @param picname  the name of the image (including the suffix).
+          * @param alpha    set this to false if you encounter transparent
+          *                 images that should be opaque. :)
+          *                 Especially for background images...
+          * @return the surface which contains the image
+          */
+        static PixMask* getMiscPicture(std::string picname, bool alpha=true);
 
     private:
         GraphicsCache();
@@ -411,8 +450,20 @@ class GraphicsCache
         //! Creates a new planted standard picture with the given parameters.
         PlantedStandardCacheItem* addPlantedStandardPic(const Player* p);
 
+        //! Creates a new port picture with the given parameters.
+        PortCacheItem* addPortPic(guint32 cityset);
+
+        //! Creates a new port picture with the given parameters.
+        SignpostCacheItem* addSignpostPic(guint32 cityset);
+
+        //! Creates a new bag-of-items picture with the given parameters.
+        BagCacheItem* addBagPic(guint32 armyset);
+
+        //! Creates a new explosion picture with the given parameters.
+        ExplosionCacheItem* addExplosionPic(guint32 tileset);
+
 	//! Creates a new new-level picture in the player's colour.
-        NewLevelCacheItem* addNewLevelPic(const Player* p);
+        NewLevelCacheItem* addNewLevelPic(const Player* p, guint32 gender);
 
         //! Creates a new flag picture with the given parameters.
         FlagCacheItem* addFlagPic(int size, const Player *p, guint32 tileset);
@@ -469,6 +520,20 @@ class GraphicsCache
 
         //! Erases the oldest planted standard cache item.
         void eraseLastPlantedStandardItem();
+
+        //! Erases the oldest port cache item.
+        void eraseLastPortItem();
+
+        //! Erases the oldest bag-of-items cache item.
+        void eraseLastBagItem();
+
+        //! Erases the oldest explosion cache item.
+        void eraseLastExplosionItem();
+
+        //! Erases the oldest signpost cache item.
+        void eraseLastSignpostItem();
+
+	//! Erase the oldest picture of a hero gaining a level.
         void eraseLastNewLevelItem();
 
         //! Erases the oldest flag cache item
@@ -518,6 +583,22 @@ class GraphicsCache
 
 	void drawTilePic(PixMask *surface, int fog_type_id, bool has_bag, bool has_standard, int standard_player_id, int stack_size, int stack_player_id, int army_type_id, bool has_tower, bool has_ship, Maptile::Building building_type, int building_subtype, Vector<int> building_tile, int building_player_id, guint32 ts, bool has_grid, guint32 tileset, guint32 cityset);
 
+        /** Loads an image
+          * 
+          * This method loads an image, adjusts it to the current resolution etc.
+          * to improve blitting performance.
+          *
+          * @note Some of the images (.jpg??) become transparent if the alpha
+          * channel is copied (they don't have transparent bits anyway), so they
+          * should be loaded with alpha set to false
+          *
+          * @param filename     full filename (with path) of the image to load
+          * @param alpha        if set to true, copy the alpha channel as well
+          *                     (i.e. if the file has transparent pixels, they
+          *                     will be marked as transparent in the returned image)
+          * @return converted image or 0 if anything failed.
+          */
+        static PixMask* loadImage(std::string filename, bool alpha = true);
         //the data
         static GraphicsCache* s_instance;
 
@@ -548,6 +629,10 @@ class GraphicsCache
         std::list<MoveBonusCacheItem*> d_movebonuslist;
         std::list<ShipCacheItem*> d_shiplist;
         std::list<PlantedStandardCacheItem*> d_plantedstandardlist;
+        std::list<PortCacheItem*> d_portlist;
+        std::list<SignpostCacheItem*> d_signpostlist;
+        std::list<BagCacheItem*> d_baglist;
+        std::list<ExplosionCacheItem*> d_explosionlist;
         std::list<NewLevelCacheItem*> d_newlevellist;
 
         //some private surfaces
@@ -560,13 +645,14 @@ class GraphicsCache
 	PixMask* d_smallinactivehero;
         PixMask* d_movebonuspic[MOVE_BONUS_TYPES];
         PixMask* d_medalpic[2][MEDAL_TYPES];
-	PixMask* d_bag;
 	PixMask* d_small_ruin_unexplored;
 	PixMask* d_small_stronghold_unexplored;
 	PixMask* d_small_ruin_explored;
 	PixMask* d_small_temple;
-	PixMask *d_newlevel;
-	PixMask *d_newlevelmask;
+	PixMask *d_newlevel_male;
+	PixMask *d_newlevelmask_male;
+	PixMask *d_newlevel_female;
+	PixMask *d_newlevelmask_female;
 };
 
 bool operator <(ArmyCacheItem lhs, ArmyCacheItem rhs);
