@@ -40,7 +40,6 @@
 #include "sound.h"
 #include "File.h"
 #include "GraphicsCache.h"
-#include "GraphicsLoader.h"
 #include "smallmap.h"
 #include "GameScenario.h"
 #include "CreateScenarioRandomize.h"
@@ -81,6 +80,7 @@
 #include "city-editor-dialog.h"
 #include "map-info-dialog.h"
 #include "new-map-dialog.h"
+#include "switch-sets-dialog.h"
 #include "itemlist-dialog.h"
 #include "rewardlist-dialog.h"
 #include "timed-message-dialog.h"
@@ -88,12 +88,14 @@
 #include "MapBackpack.h"
 
 
-MainWindow::MainWindow()
+MainWindow::MainWindow(std::string load_filename)
 {
+  d_load_filename = load_filename;
   bigmap = NULL;
   smallmap = NULL;
   game_scenario = NULL;
   d_create_scenario_names = NULL;
+  needs_saving = false;
     Glib::RefPtr<Gtk::Builder> xml
 	= Gtk::Builder::create_from_file(get_glade_path() + "/main-window.ui");
 
@@ -170,14 +172,17 @@ MainWindow::MainWindow()
 			      EditorBigMap::CITY, 1);
     setup_pointer_radiobutton(xml, "erase", "button_erase",
 			      EditorBigMap::ERASE, 1);
+    setup_pointer_radiobutton(xml, "move", "button_move",
+			      EditorBigMap::MOVE, 1);
     setup_pointer_radiobutton(xml, "draw_port", "button_port",
 			      EditorBigMap::PORT, 1);
     setup_pointer_radiobutton(xml, "draw_bridge", "button_bridge",
 			      EditorBigMap::BRIDGE, 1);
     setup_pointer_radiobutton(xml, "draw_bag", "button_bag",
 			      EditorBigMap::BAG, 1);
-    on_pointer_radiobutton_toggled();
 
+    xml->get_widget("players_hbox", players_hbox);
+    on_pointer_radiobutton_toggled();
 
     xml->get_widget("mouse_position_label", mouse_position_label);
     
@@ -195,6 +200,9 @@ MainWindow::MainWindow()
     xml->get_widget("save_map_as_menuitem", save_map_as_menuitem);
     save_map_as_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &MainWindow::on_save_map_as_activated));
+    xml->get_widget("import_map_from_sav_menuitem", import_map_from_sav_menuitem);
+    import_map_from_sav_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_import_map_activated));
     xml->get_widget("export_as_bitmap_menuitem", export_as_bitmap_menuitem);
     export_as_bitmap_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &MainWindow::on_export_as_bitmap_activated));
@@ -228,6 +236,9 @@ MainWindow::MainWindow()
     xml->get_widget("smooth_map_menuitem", smooth_map_menuitem);
     smooth_map_menuitem->signal_activate().connect
       (sigc::mem_fun(this, &MainWindow::on_smooth_map_activated));
+    xml->get_widget("switch_sets_menuitem", switch_sets_menuitem);
+    switch_sets_menuitem->signal_activate().connect
+      (sigc::mem_fun(this, &MainWindow::on_switch_sets_activated));
     xml->get_widget("smooth_screen_menuitem", smooth_screen_menuitem);
     smooth_screen_menuitem->signal_activate().connect
       (sigc::mem_fun (this, &MainWindow::on_smooth_screen_activated));
@@ -364,8 +375,7 @@ void MainWindow::on_bigmap_surface_changed(Gtk::Allocation box)
   if (box.get_width() != last_box.get_width() || box.get_height() != last_box.get_height())
     {
       bigmap->screen_size_changed(bigmap_drawingarea->get_allocation());
-      bigmap->draw();
-      smallmap->draw(Playerlist::getActiveplayer());
+      redraw();
     }
   last_box = box;
 }
@@ -398,11 +408,13 @@ bool MainWindow::on_smallmap_exposed(GdkEventExpose *event)
 
 void MainWindow::init()
 {
-  GraphicsLoader::instantiateImages(Armysetlist::getInstance());
-  GraphicsLoader::instantiateImages(Tilesetlist::getInstance());
-  GraphicsLoader::instantiateImages(Shieldsetlist::getInstance());
-  GraphicsLoader::instantiateImages(Citysetlist::getInstance());
+  Armysetlist::getInstance()->instantiateImages();
+  Tilesetlist::getInstance()->instantiateImages();
+  Shieldsetlist::getInstance()->instantiateImages();
+  Citysetlist::getInstance()->instantiateImages();
   show_initial_map();
+  Playerlist::getInstance()->setActiveplayer(Playerlist::getInstance()->getNeutral());
+  fill_players();
 }
 
 void MainWindow::hide()
@@ -412,18 +424,45 @@ void MainWindow::hide()
 
 bool MainWindow::on_delete_event(GdkEventAny *e)
 {
-    hide();
-    
-    return true;
+  return !quit();
 }
 
 void MainWindow::show_initial_map()
 {
-    set_filled_map(112, 156, Tile::WATER, "default", "default", "default",
-		   "default");
-    setup_terrain_radiobuttons();
-    remove_tile_style_buttons();
-    setup_tile_style_buttons(Tile::GRASS);
+  if (d_load_filename.empty() == false)
+    {
+      clear_map_state();
+
+      bool broken;
+      if (game_scenario)
+	delete game_scenario;
+      current_save_filename = d_load_filename;
+      game_scenario = new GameScenario(current_save_filename, broken);
+      if (d_create_scenario_names)
+	delete d_create_scenario_names;
+      d_create_scenario_names = new CreateScenarioRandomize();
+      if (broken == false)
+	{
+	  init_map_state();
+	  bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+          setup_terrain_radiobuttons();
+          remove_tile_style_buttons();
+          setup_tile_style_buttons(Tile::GRASS);
+	}
+      else
+	{
+	  d_load_filename = "";
+	  show_initial_map();
+	}
+    }
+  else
+    {
+      set_filled_map(112, 156, Tile::WATER, "default", "default", "default",
+		     "default");
+      setup_terrain_radiobuttons();
+      remove_tile_style_buttons();
+      setup_tile_style_buttons(Tile::GRASS);
+    }
 }
 
 void MainWindow::set_filled_map(int width, int height, int fill_style, std::string tileset, std::string shieldset, std::string cityset, std::string armyset)
@@ -456,11 +495,12 @@ void MainWindow::set_filled_map(int width, int height, int fill_style, std::stri
       Armysetlist::getInstance()->getArmyset(armyset)->getId();
     Shieldsetlist *ssl = Shieldsetlist::getInstance();
     Shieldset *ss = ssl->getShieldset(shieldset);
-    Player* neutral = new AI_Dummy(_("Neutral"), armyset_id, 
+    std::string name = d_create_scenario_names->getPlayerName(Shield::NEUTRAL);
+    Player* neutral = new AI_Dummy(name, armyset_id, 
 				   ssl->getColor(ss->getId(), MAX_PLAYERS), 
 				   width, height, MAX_PLAYERS);
     neutral->setType(Player::AI_DUMMY);
-    Playerlist::getInstance()->push_back(neutral);
+    Playerlist::getInstance()->add(neutral);
     Playerlist::getInstance()->setNeutral(neutral);
     Playerlist::getInstance()->nextPlayer();
 
@@ -502,13 +542,15 @@ void MainWindow::set_random_map(int width, int height,
     // creation...
     guint32 armyset_id = 
       Armysetlist::getInstance()->getArmyset(armyset)->getId();
+    Citysetlist::getInstance();
     Shieldsetlist *ssl = Shieldsetlist::getInstance();
     Shieldset *ss = ssl->getShieldset(shieldset);
-    Player* neutral = new AI_Dummy(_("Neutral"), armyset_id, 
+    std::string name = d_create_scenario_names->getPlayerName(Shield::NEUTRAL);
+    Player* neutral = new AI_Dummy(name, armyset_id, 
 				   ssl->getColor(ss->getId(), MAX_PLAYERS), 
 				   width, height, MAX_PLAYERS);
     neutral->setType(Player::AI_DUMMY);
-    Playerlist::getInstance()->push_back(neutral);
+    Playerlist::getInstance()->add(neutral);
     Playerlist::getInstance()->setNeutral(neutral);
     Playerlist::getInstance()->nextPlayer();
     
@@ -537,7 +579,12 @@ void MainWindow::set_random_map(int width, int height,
     
     gen.setPercentages(water, forest, swamp, hills, mountains);
     
+    gen.setCityset(GameMap::getInstance()->getCityset());
     gen.makeMap(width, height, false);
+    GameMap::deleteInstance();
+    GameMap::setWidth(width);
+    GameMap::setHeight(height);
+    GameMap::getInstance(tileset, shieldset, cityset);
     GameMap::getInstance()->fill(&gen);
 
     Itemlist::createStandardInstance();
@@ -549,6 +596,7 @@ void MainWindow::set_random_map(int width, int height,
       delete d_create_scenario_names;
     d_create_scenario_names = new CreateScenarioRandomize();
     
+    Cityset *cs = Citysetlist::getInstance()->getCityset(cityset);
     // now fill the lists
     const Maptile::Building* build = gen.getBuildings(width, height);
     for (int j = 0; j < height; j++)
@@ -556,15 +604,18 @@ void MainWindow::set_random_map(int width, int height,
 	    switch(build[j * width + i])
 	    {
 	    case Maptile::CITY:
-		Citylist::getInstance()->add(new City(Vector<int>(i,j)));
+		Citylist::getInstance()->add
+		  (new City(Vector<int>(i,j), cs->getCityTileWidth()));
 		(*Citylist::getInstance()->rbegin())->setOwner(
 		    Playerlist::getInstance()->getNeutral());
 		break;
 	    case Maptile::TEMPLE:
-		Templelist::getInstance()->add(new Temple(Vector<int>(i,j)));
+		Templelist::getInstance()->add
+		  (new Temple(Vector<int>(i,j), cs->getTempleTileWidth()));
 		break;
 	    case Maptile::RUIN:
-		Ruinlist::getInstance()->add(new Ruin(Vector<int>(i,j)));
+		Ruinlist::getInstance()->add
+		  (new Ruin(Vector<int>(i,j), cs->getRuinTileWidth()));
 		break;
 	    case Maptile::SIGNPOST:
 		Signpostlist::getInstance()->add(new Signpost(Vector<int>(i,j)));
@@ -628,6 +679,7 @@ bool MainWindow::on_bigmap_mouse_button_event(GdkEventButton *e)
 	bigmap->mouse_button_event(to_input_event(e));
 	if (smallmap)
 	  smallmap->draw(Playerlist::getActiveplayer());
+	needs_saving = true;
     }
     
     return true;
@@ -719,6 +771,7 @@ void MainWindow::on_new_map_activated()
 	    set_filled_map(d.map.width, d.map.height, d.map.fill_style, 
 			   d.map.tileset, d.map.shieldset, d.map.cityset,
 			   d.map.armyset);
+        needs_saving = true;
     }
 }
 
@@ -771,6 +824,7 @@ void MainWindow::on_save_map_activated()
 	on_save_map_as_activated();
     else
     {
+        needs_saving = false;
 	bool success = game_scenario->saveGame(current_save_filename, "map");
 	if (!success)
 	    show_error(_("Map was not saved!"));
@@ -857,31 +911,64 @@ void MainWindow::on_save_map_as_activated()
 	bool success = game_scenario->saveGame(current_save_filename, "map");
 	if (!success)
 	    show_error(_("Map was not saved!"));
+        needs_saving = false;
     }
 }
 
+bool MainWindow::quit()
+{
+  if (needs_saving)
+    {
+      Gtk::Dialog* dialog;
+      Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path() + 
+					 "/editor-quit-dialog.ui");
+      xml->get_widget("dialog", dialog);
+      dialog->set_transient_for(*window);
+      int response = dialog->run();
+      dialog->hide();
+      delete dialog;
+      
+      if (response == Gtk::RESPONSE_CANCEL) //we don't want to quit
+	return false;
+
+      else if (response == Gtk::RESPONSE_ACCEPT) // save and quit
+	on_save_map_activated();
+      //else if (Response == Gtk::CLOSE) // don't save just quit
+      window->hide();
+    }
+  else
+    window->hide();
+  return true;
+}
+
 void MainWindow::on_quit_activated()
 {
-    // FIXME: ask
-    bool end = true;
-
-    if (end) {
-    }
-    window->hide();
+  quit();
 }
 
 void MainWindow::on_edit_players_activated()
 {
-    PlayersDialog d(d_width, d_height);
+    PlayersDialog d(d_create_scenario_names, d_width, d_height);
     d.set_parent_window(*window);
-    d.run();
+    Player *active = Playerlist::getActiveplayer();
+    int response = d.run();
+    if (response == Gtk::RESPONSE_ACCEPT)
+      {
+	if (Playerlist::getInstance()->getPlayer(active->getId()))
+	  Playerlist::getInstance()->setActiveplayer(active);
+	needs_saving = true;
+	fill_players();
+      }
 }
 
 void MainWindow::on_edit_map_info_activated()
 {
     MapInfoDialog d(game_scenario);
     d.set_parent_window(*window);
-    d.run();
+    int response = d.run();
+    if (response == Gtk::RESPONSE_ACCEPT)
+      needs_saving = true;
 }
 
 void MainWindow::on_fullscreen_activated()
@@ -895,7 +982,7 @@ void MainWindow::on_fullscreen_activated()
 void MainWindow::on_tile_graphics_toggled()
 {
   bigmap->toggleViewStylesOrTypes();
-  bigmap->draw();
+  bigmap->draw(Playerlist::getViewingplayer());
 }
 void MainWindow::on_grid_toggled()
 {
@@ -1042,6 +1129,9 @@ void MainWindow::on_pointer_radiobutton_toggled()
 			    get_tile_style_id());
     terrain_type_table->set_sensitive(pointer == EditorBigMap::TERRAIN);
     terrain_tile_style_hbox->set_sensitive(pointer == EditorBigMap::TERRAIN);
+
+    players_hbox->set_sensitive (pointer == EditorBigMap::STACK || 
+				 pointer == EditorBigMap::CITY);
 }
 
 Tile::Type MainWindow::get_terrain()
@@ -1109,6 +1199,14 @@ void MainWindow::on_smallmap_changed(Glib::RefPtr<Gdk::Pixmap> map, Gdk::Rectang
 
 void MainWindow::init_maps()
 {
+    // init the smallmap
+    if (smallmap)
+      delete smallmap;
+    smallmap =new SmallMap;
+    smallmap->resize();
+    smallmap->map_changed.connect(
+	sigc::mem_fun(this, &MainWindow::on_smallmap_changed));
+
     // init the bigmap
     if (bigmap)
       delete bigmap;
@@ -1119,16 +1217,10 @@ void MainWindow::init_maps()
 	sigc::mem_fun(this, &MainWindow::on_objects_selected));
     bigmap->map_changed.connect(
 	sigc::mem_fun(this, &MainWindow::on_bigmap_changed));
+
     // grid is on by default
     bigmap->toggle_grid();
     
-    // init the smallmap
-    if (smallmap)
-      delete smallmap;
-    smallmap =new SmallMap;
-    smallmap->map_changed.connect(
-	sigc::mem_fun(this, &MainWindow::on_smallmap_changed));
-
     // connect the two maps
     bigmap->view_changed.connect(
 	sigc::mem_fun(smallmap, &SmallMap::set_view));
@@ -1137,7 +1229,8 @@ void MainWindow::init_maps()
     smallmap->view_changed.connect(
 	sigc::mem_fun(bigmap, &EditorBigMap::set_view));
 
-    smallmap->resize(GameMap::get_dim() * 2);
+    //trigger the bigmap to resize the view box in the smallmap
+    bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
 }
 
 void MainWindow::on_mouse_on_tile(Vector<int> tile)
@@ -1195,47 +1288,57 @@ void MainWindow::popup_dialog_for_object(UniquelyIdentified *object)
     {
 	StackEditorDialog d(o);
 	d.set_parent_window(*window);
-	d.run();
+	int response = d.run();
+	if (response == Gtk::RESPONSE_ACCEPT)
+	  needs_saving = true;
 
 	// we might have changed something visible
-	bigmap->draw();
-	smallmap->draw(Playerlist::getActiveplayer());
+	redraw();
     }
     else if (City *o = dynamic_cast<City *>(object))
     {
 	CityEditorDialog d(o, d_create_scenario_names);
 	d.set_parent_window(*window);
-	d.run();
+	int response = d.run();
+	if (response == Gtk::RESPONSE_ACCEPT)
+	  needs_saving = true;
 
 	// we might have changed something visible
-	bigmap->draw();
-	smallmap->draw(Playerlist::getActiveplayer());
+	redraw();
     }
     else if (Ruin *o = dynamic_cast<Ruin *>(object))
     {
 	RuinEditorDialog d(o, d_create_scenario_names);
 	d.set_parent_window(*window);
-	d.run();
+	int response = d.run();
+	if (response == Gtk::RESPONSE_ACCEPT)
+	  needs_saving = true;
     }
     else if (Signpost *o = dynamic_cast<Signpost *>(object))
     {
 	SignpostEditorDialog d(o, d_create_scenario_names);
 	d.set_parent_window(*window);
-	d.run();
+	int response = d.run();
+	if (response == Gtk::RESPONSE_ACCEPT)
+	  needs_saving = true;
     }
     else if (Temple *o = dynamic_cast<Temple *>(object))
     {
 	TempleEditorDialog d(o, d_create_scenario_names);
 	d.set_parent_window(*window);
-	d.run();
+	int response = d.run();
+	if (response == Gtk::RESPONSE_ACCEPT)
+	  needs_saving = true;
 
 	// we might have changed something visible
-	bigmap->draw();
+	redraw();
     }
     else if (MapBackpack *b = dynamic_cast<MapBackpack*>(object))
       {
 	BackpackEditorDialog d(b);
-	d.run();
+	int response = d.run();
+	if (response == Gtk::RESPONSE_ACCEPT)
+	  needs_saving = true;
       }
 }
 
@@ -1243,7 +1346,7 @@ void MainWindow::on_smooth_map_activated()
 {
   GameMap::getInstance()->applyTileStyles(0, 0, GameMap::getHeight(), 
 					  GameMap::getWidth(), true);
-  bigmap->draw();
+  redraw();
 }
 
 void MainWindow::on_smooth_screen_activated()
@@ -1255,14 +1358,18 @@ void MainWindow::on_edit_items_activated()
 {
   ItemlistDialog d;
   d.set_parent_window(*window);
-  d.run();
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    needs_saving = true;
 }
 
 void MainWindow::on_edit_rewards_activated()
 {
   RewardlistDialog d;
   d.set_parent_window(*window);
-  d.run();
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    needs_saving = true;
 }
 
 void MainWindow::randomize_city(City *c)
@@ -1278,6 +1385,7 @@ void MainWindow::on_random_all_cities_activated()
   Citylist *cl = Citylist::getInstance();
   for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
     randomize_city(*it);
+  needs_saving = true;
 }
 
 void MainWindow::on_random_unnamed_cities_activated()
@@ -1288,6 +1396,7 @@ void MainWindow::on_random_unnamed_cities_activated()
       if ((*it)->isUnnamed() == true)
 	randomize_city(*it);
     }
+  needs_saving = true;
 }
 
 void MainWindow::randomize_ruin(Ruin *r)
@@ -1306,6 +1415,7 @@ void MainWindow::on_random_all_ruins_activated()
   Ruinlist *rl = Ruinlist::getInstance();
   for (Ruinlist::iterator it = rl->begin(); it != rl->end(); it++)
     randomize_ruin(*it);
+  needs_saving = true;
 }
 
 void MainWindow::on_random_unnamed_ruins_activated()
@@ -1316,6 +1426,7 @@ void MainWindow::on_random_unnamed_ruins_activated()
       if ((*it)->isUnnamed() == true)
 	randomize_ruin(*it);
     }
+  needs_saving = true;
 }
 
 void MainWindow::on_random_all_temples_activated()
@@ -1332,6 +1443,7 @@ void MainWindow::on_random_all_temples_activated()
 	  renamable_temple->setName(name);
 	}
     }
+  needs_saving = true;
 }
 
 void MainWindow::on_random_unnamed_temples_activated()
@@ -1351,6 +1463,7 @@ void MainWindow::on_random_unnamed_temples_activated()
 	    }
 	}
     }
+  needs_saving = true;
 }
 
 void MainWindow::randomize_signpost(Signpost *signpost)
@@ -1370,6 +1483,7 @@ void MainWindow::on_random_all_signs_activated()
   Signpostlist *sl = Signpostlist::getInstance();
   for (Signpostlist::iterator it = sl->begin(); it != sl->end(); it++)
     randomize_signpost(*it);
+  needs_saving = true;
 }
 
 void MainWindow::on_random_unnamed_signs_activated()
@@ -1380,6 +1494,7 @@ void MainWindow::on_random_unnamed_signs_activated()
       if ((*it)->getName() == DEFAULT_SIGNPOST)
 	randomize_signpost(*it);
     }
+  needs_saving = true;
 }
 
 void MainWindow::on_help_about_activated()
@@ -1394,7 +1509,7 @@ void MainWindow::on_help_about_activated()
   dialog->set_icon_from_file(File::getMiscFile("various/tileset_icon.png"));
 
   dialog->set_version(PACKAGE_VERSION);
-  dialog->set_logo(GraphicsLoader::getMiscPicture("castle_icon.png")->to_pixbuf());
+  dialog->set_logo(GraphicsCache::getMiscPicture("castle_icon.png")->to_pixbuf());
   dialog->show_all();
   dialog->run();
   delete dialog;
@@ -1426,3 +1541,130 @@ void MainWindow::on_validate_activated()
   dialog.run();
   dialog.hide();
 }
+      
+	
+void MainWindow::clear_save_file_of_scenario_specific_data()
+{
+  Playerlist *plist = Playerlist::getInstance();
+  for (Playerlist::iterator i = plist->begin(); i != plist->end(); i++)
+    {
+      (*i)->clearActionlist();
+      (*i)->clearHistorylist();
+      (*i)->clearStacklist();
+      (*i)->clearFogMap();
+      (*i)->setGold(1000);
+      (*i)->revive();
+    }
+}
+
+void MainWindow::on_import_map_activated()
+{
+    Gtk::FileChooserDialog chooser(*window, _("Choose Game to Load Map from"));
+    Gtk::FileFilter sav_filter;
+    sav_filter.add_pattern("*.sav");
+    chooser.set_filter(sav_filter);
+    chooser.set_current_folder(File::getSavePath());
+
+    chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+    chooser.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
+    chooser.set_default_response(Gtk::RESPONSE_ACCEPT);
+	
+    chooser.show_all();
+    int res = chooser.run();
+    
+    if (res == Gtk::RESPONSE_ACCEPT)
+    {
+        std::string filename = chooser.get_filename();
+	chooser.hide();
+
+	clear_map_state();
+
+	bool broken;
+	if (game_scenario)
+	  delete game_scenario;
+	game_scenario = new GameScenario(filename, broken);
+
+	if (broken)
+	{
+	    show_error(String::ucompose(_("Could not load game %1."),
+					filename));
+	    current_save_filename = "";
+	    return;
+	}
+
+	if (d_create_scenario_names)
+	  delete d_create_scenario_names;
+	d_create_scenario_names = new CreateScenarioRandomize();
+
+	//now lets get rid of stuff.
+	clear_save_file_of_scenario_specific_data();
+
+	init_map_state();
+	bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+    }
+}
+      
+void MainWindow::redraw()
+{
+  bigmap->draw(Playerlist::getViewingplayer());
+  smallmap->draw(Playerlist::getActiveplayer());
+}
+
+void MainWindow::on_switch_sets_activated()
+{
+  SwitchSetsDialog d;
+  int response = d.run();
+  if (response == Gtk::RESPONSE_ACCEPT)
+    {
+      needs_saving = true;
+      bigmap->screen_size_changed(bigmap_drawingarea->get_allocation()); 
+      redraw();
+      fill_players();
+    }
+}
+    
+	
+void MainWindow::on_player_toggled(PlayerItem item)
+{
+  Player *p = Playerlist::getInstance()->getPlayer(item.player_id);
+  if (p)
+    Playerlist::getInstance()->setActiveplayer(p);
+  fill_players();
+}
+
+void MainWindow::fill_players()
+{
+  for (std::list<PlayerItem>::iterator it = player_buttons.begin();
+       it != player_buttons.end(); it++)
+    players_hbox->remove(dynamic_cast<Gtk::Widget&>(*(*it).button));
+  player_buttons.clear();
+  Playerlist *pl = Playerlist::getInstance();
+  bool sensitive = players_hbox->get_sensitive();
+  if (!sensitive)
+    players_hbox->set_sensitive(true);
+  for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
+    {
+      Gtk::ToggleButton *toggle = new Gtk::ToggleButton();
+      toggle->foreach(sigc::mem_fun(toggle, &Gtk::Container::remove));
+
+      Gtk::Image *image = new Gtk::Image();
+      image->property_pixbuf() = 
+	GraphicsCache::getInstance()->getShieldPic(1, *it)->to_pixbuf();
+      toggle->add(*manage(image));
+      toggle->show_all();
+      if (*it == pl->getActiveplayer())
+	toggle->set_active(true);
+
+      struct PlayerItem item;
+      item.button = toggle;
+      item.player_id = (*it)->getId();
+      player_buttons.push_back(item);
+      toggle->set_tooltip_text((*it)->getName());
+      players_hbox->pack_start(*manage(toggle), Gtk::PACK_SHRINK);
+      toggle->signal_toggled().connect
+	(sigc::bind(sigc::mem_fun(this, &MainWindow::on_player_toggled), item));
+    }
+  players_hbox->show_all();
+  if (!sensitive)
+    players_hbox->set_sensitive(false);
+}
