@@ -1,8 +1,8 @@
-//  Copyright (C) 2008 Ben Asselstine
+//  Copyright (C) 2008, 2009 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 
 #include <config.h>
 
-#include <libglademm/xml.h>
+#include <gtkmm.h>
 #include <sigc++/functors/mem_fun.h>
 
 #include "tile-preview-dialog.h"
@@ -29,15 +29,13 @@
 #include "File.h"
 
 
-TilePreviewDialog::TilePreviewDialog(Tile *tile, Uint32 tileSize)
+TilePreviewDialog::TilePreviewDialog(Tile *tile, guint32 tileSize)
 {
-    Glib::RefPtr<Gnome::Glade::Xml> xml
-	= Gnome::Glade::Xml::create(get_glade_path()
-				    + "/tile-preview-dialog.glade");
+    Glib::RefPtr<Gtk::Builder> xml
+	= Gtk::Builder::create_from_file(get_glade_path()
+				    + "/tile-preview-dialog.ui");
 
-    Gtk::Dialog *d = 0;
-    xml->get_widget("dialog", d);
-    dialog.reset(d);
+    xml->get_widget("dialog", dialog);
 
     xml->get_widget("next_button", next_button);
     next_button->signal_clicked().connect
@@ -48,18 +46,20 @@ TilePreviewDialog::TilePreviewDialog(Tile *tile, Uint32 tileSize)
     xml->get_widget("refresh_button", refresh_button);
     refresh_button->signal_clicked().connect
       (sigc::mem_fun(this, &TilePreviewDialog::on_refresh_clicked));
-    xml->get_widget("scene_box", scene_box);
+    xml->get_widget("preview_image", preview_image);
 
-    std::vector<Glib::RefPtr<Gdk::Pixbuf> > base_tilestyles;
+    std::vector<PixMask* > base_tilestyles;
     base_tilestyles = 
       disassemble_row(File::getMiscFile("various/editor/tilestyles.png"), 17);
 
     d_tileSize = tileSize;
-    std::vector<Glib::RefPtr<Gdk::Pixbuf> >::iterator it;
+    std::vector<PixMask* >::iterator it;
     for (it = base_tilestyles.begin(); it != base_tilestyles.end(); it++)
-      tilestyle_images.push_back((*it)->scale_simple((int)d_tileSize, 
-						     (int)d_tileSize, 
-						     Gdk::INTERP_BILINEAR));
+      {
+	PixMask *pix = (*it)->copy();
+	PixMask::scale(pix, d_tileSize, d_tileSize);
+	tilestyle_images.push_back(pix);
+      }
 
     std::string scene;
     TilePreviewScene *s;
@@ -144,7 +144,6 @@ TilePreviewDialog::TilePreviewDialog(Tile *tile, Uint32 tileSize)
 	scenes.push_back(s);
 	break;
       }
-    scene_table = NULL;
     d_tile = tile;
     current_scene = scenes.begin();
     if (*current_scene)
@@ -153,6 +152,10 @@ TilePreviewDialog::TilePreviewDialog(Tile *tile, Uint32 tileSize)
     update_buttons();
 }
 
+TilePreviewDialog::~TilePreviewDialog()
+{
+  delete dialog;
+}
 void TilePreviewDialog::set_parent_window(Gtk::Window &parent)
 {
     dialog->set_transient_for(parent);
@@ -210,8 +213,7 @@ void TilePreviewDialog::update_scene(TilePreviewScene *scene)
 {
   if (!scene)
     return;
-  if (scene_table)
-    delete scene_table;
+  /*
   scene_table = new Gtk::Table(scene->getHeight(), scene->getWidth());
   scene_box->add(*manage(scene_table));
   for (int i = 0; i < scene->getHeight(); i++)
@@ -232,7 +234,9 @@ void TilePreviewDialog::update_scene(TilePreviewScene *scene)
 	  image->show();
 	}
     }
-  scene_table->show_all();
+    */
+  preview_image->property_pixbuf() = scene->renderScene(d_tileSize);
+  preview_image->show_all();
 }
 
 void TilePreviewDialog::update_buttons()
