@@ -27,6 +27,7 @@
 #include <gtkmm.h>
 
 #include "army-info-tip.h"
+#include "city-info-tip.h"
 #include "stack-info-tip.h"
 #include "decorated.h"
 
@@ -53,6 +54,7 @@ class City;
 class Reward;
 class Reward_Ruin;
 class NextTurn;
+class LocationBox;
 
 /** The main window in which all the game action is going on.
   *
@@ -118,6 +120,7 @@ class GameWindow: public Decorated
     Gtk::MenuItem *signpost_menuitem;
     Gtk::MenuItem *diplomacy_report_menuitem;
     Gtk::MenuItem *city_history_menuitem;
+    Gtk::MenuItem *ruin_history_menuitem;
     Gtk::MenuItem *event_history_menuitem;
     Gtk::MenuItem *gold_history_menuitem;
     Gtk::MenuItem *winner_history_menuitem;
@@ -149,6 +152,7 @@ class GameWindow: public Decorated
     Gtk::MenuItem *quests_menuitem;
     Gtk::MenuItem *preferences_menuitem;
     Gtk::Box *stack_info_box;
+    Gtk::Table *stack_info_button_table; //not loaded from .ui
     Gtk::Box *stack_info_container;
     Gtk::Label *group_moves_label;
     Gtk::Image *terrain_image;
@@ -189,9 +193,12 @@ class GameWindow: public Decorated
 
     Stack *currently_selected_stack;
     ArmyInfoTip* army_info_tip;
+    CityInfoTip* city_info_tip;
     StackInfoTip* stack_info_tip;
     typedef std::vector<Gtk::ToggleButton *> army_buttons_type;
     army_buttons_type army_buttons;
+    typedef std::vector<Gtk::RadioButton*> stack_buttons_type;
+    stack_buttons_type stack_buttons;
     Gtk::EventBox *map_eventbox;
     Gtk::EventBox *bigmap_eventbox;
     Gtk::ToggleButton *group_ungroup_toggle;
@@ -247,6 +254,7 @@ class GameWindow: public Decorated
     void on_ruin_report_activated();
     void on_army_bonus_activated();
     void on_city_history_activated();
+    void on_ruin_history_activated();
     void on_event_history_activated();
     void on_gold_history_activated();
     void on_winner_history_activated();
@@ -259,15 +267,14 @@ class GameWindow: public Decorated
     // info pane at the bottom
     void show_stats();
     void show_progress();
-    void show_stack(Stack *s);
-    void fill_in_group_info (Stack *s);
-    void on_army_toggled(Gtk::ToggleButton *toggle, Army *army);
+    void show_stack(StackTile *s);
+    void fill_in_group_info (StackTile *stile, Stack *s);
+    void on_army_toggled(Gtk::ToggleButton *toggle, Stack *stack, Army *army);
+    void on_stack_toggled(Gtk::RadioButton *radio, Stack *stack);
     void on_group_toggled(Gtk::ToggleButton *toggle);
     bool on_army_button_event(GdkEventButton *e,
 			      Gtk::ToggleButton *toggle, Army *army);
     void clear_army_buttons();
-    void update_army_buttons();
-    void ensure_one_army_button_active();
 
     // shield set on the top
     void show_shield_turn();
@@ -282,18 +289,20 @@ class GameWindow: public Decorated
     void on_bigmap_changed(Glib::RefPtr<Gdk::Pixmap> map);
     void on_stack_info_changed(Stack *s);
     void on_bigmap_tip_changed(Glib::ustring tip, MapTipPosition pos);
-    void on_stack_tip_changed(Stack *s, MapTipPosition pos);
+    void on_stack_tip_changed(StackTile *stile, MapTipPosition pos);
+    void on_city_tip_changed(City *city, MapTipPosition pos);
     void on_ruin_searched(Ruin *ruin, Stack *s, Reward *reward);
-    void on_sage_visited(Ruin *ruin, Stack *s);
+    Reward* on_sage_visited(Ruin *ruin, Sage *sage, Stack *s);
     void on_ruin_rewarded(Reward_Ruin *reward);
-    void on_fight_started(Fight &fight);
+    void on_fight_started(LocationBox box, Fight &fight);
+    void on_abbreviated_fight_started(LocationBox box);
     void on_ruinfight_started(Stack *attackers, Stack *defenders);
     void on_ruinfight_finished(Fight::Result result);
     bool on_hero_offers_service(Player *player, HeroProto *hero, City *city, int gold);
     bool on_enemy_offers_surrender(int numEnemies);
     void on_surrender_answered (bool accepted);
     bool on_stack_considers_treachery (Player *me, Stack *stack, Player *them, Vector<int> pos);
-    bool on_temple_searched(bool hasHero, Temple *temple, int blessCount);
+    bool on_temple_searched(Hero *hero, Temple *temple, int blessCount);
     void on_quest_assigned(Hero *hero, Quest *quest);
     CityDefeatedAction on_city_defeated(City *city, int gold);
     void on_city_pillaged(City *city, int gold, int pillaged_army_type);
@@ -306,12 +315,14 @@ class GameWindow: public Decorated
     void on_remote_next_player_turn();
     void on_hero_brings_allies(int alliesCount);
     void on_medal_awarded_to_army(Army *army, int medaltype);
-    Army::Stat on_army_gains_level(Army *army);
+    Army::Stat on_hero_gains_level(Hero *hero);
     void on_game_loaded(Player *player);
     void on_game_over(Player *winner);
     void on_player_died(Player *player);
     void on_advice_asked(float percent);
+    void on_stack_moves(Stack *stack, Vector<int> pos);
     void end_turn_play_by_mail ();
+    void on_commentator_comments(std::string comment);
 
     // quest manager callbacks
     void on_quest_completed(Quest *quest, Reward *reward);
