@@ -1,6 +1,6 @@
 // Copyright (C) 2003 Michael Bartl
 // Copyright (C) 2003, 2004, 2005, 2006 Ulf Lorenz
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #include <iostream>
 #include "tileset.h"
 #include "MapBackpack.h"
+#include "stacktile.h"
 
 Maptile::Maptile(Tileset* tileSet, int x, int y, guint32 type, TileStyle *tileStyle)
     :d_index(type), d_building(NONE) 
@@ -30,6 +31,7 @@ Maptile::Maptile(Tileset* tileSet, int x, int y, guint32 type, TileStyle *tileSt
     d_tileSet = tileSet;
     d_tileStyle = tileStyle;
     d_backpack = new MapBackpack(Vector<int>(x,y));
+    d_stacktile = new StackTile(Vector<int>(x,y));
 }
 
 Maptile::Maptile(Tileset* tileSet, int x, int y, Tile::Type type, TileStyle *tileStyle)
@@ -50,6 +52,7 @@ Maptile::Maptile(Tileset* tileSet, int x, int y, Tile::Type type, TileStyle *til
     if (found == false)
       d_index = 0;
     d_backpack = new MapBackpack(Vector<int>(x,y));
+    d_stacktile = new StackTile(Vector<int>(x,y));
 }
 
 Maptile::~Maptile()
@@ -84,12 +87,18 @@ void Maptile::printDebugInfo() const
 
 bool Maptile::isCityTerrain()
 {
-  if (getBuilding() != Maptile::CITY || getBuilding() != Maptile::RUIN ||
-       getBuilding() != Maptile::TEMPLE)
+  if (getBuilding() == Maptile::CITY || getBuilding() == Maptile::RUIN ||
+      getBuilding() == Maptile::TEMPLE)
     return true;
   return false;
 }
 
+bool Maptile::isRoadTerrain()
+{
+  if (getBuilding() == Maptile::ROAD || getBuilding() == Maptile::BRIDGE)
+    return true;
+  return false;
+}
 bool Maptile::isOpenTerrain()
 {
   if (isCityTerrain())
