@@ -1,5 +1,5 @@
 // Copyright (C) 2006, 2007 Ulf Lorenz
-// Copyright (C) 2006, 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2006, 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -39,7 +39,9 @@
 
 OverviewMap::OverviewMap()
 {
-    d_player = Playerlist::getActiveplayer();
+  blank_screen = false;
+  map_tiles_per_tile = GameMap::calculateTilesPerOverviewMapTile();
+  pixels_per_tile = 2.0;
 }
 
 OverviewMap::~OverviewMap()
@@ -303,25 +305,31 @@ void OverviewMap::draw_terrain_tile(Maptile *t, int i, int j)
 		     i, j, shadowed);
 }
 
-int OverviewMap::calculateResizeFactor()
+int OverviewMap::calculatePixelsPerTile(int width, int height)
 {
-  if (GameMap::getWidth() <= (int)MAP_SIZE_TINY_WIDTH && 
-      GameMap::getHeight() <= (int)MAP_SIZE_TINY_HEIGHT)
+  if (width <= (int)MAP_SIZE_TINY_WIDTH && 
+      height <= (int)MAP_SIZE_TINY_HEIGHT)
     return 4;
-  else if (GameMap::getWidth() <= (int)MAP_SIZE_SMALL_WIDTH && 
-	   GameMap::getHeight() <= (int)MAP_SIZE_SMALL_HEIGHT)
+  else if (width <= (int)MAP_SIZE_SMALL_WIDTH && 
+	   height <= (int)MAP_SIZE_SMALL_HEIGHT)
     return 3;
   else
     return 2;
 }
 
-void OverviewMap::resize()
+int OverviewMap::calculatePixelsPerTile()
 {
-  int factor = calculateResizeFactor();
-  resize(GameMap::get_dim() * factor);
+  return calculatePixelsPerTile(GameMap::getWidth(), GameMap::getHeight());
 }
 
-void OverviewMap::resize(Vector<int> max_dimensions)
+void OverviewMap::resize()
+{
+  int factor = calculatePixelsPerTile();
+  resize(GameMap::get_dim() * factor, 
+         GameMap::calculateTilesPerOverviewMapTile());
+}
+
+void OverviewMap::resize(Vector<int> max_dimensions, float scale)
 {
   surface.reset();
 
@@ -341,6 +349,7 @@ void OverviewMap::resize(Vector<int> max_dimensions)
 	d.x = int(round(bigmap_dim.x * pixels_per_tile));
 	d.y = max_dimensions.y;
     }
+    map_tiles_per_tile = scale;
 
     
     static_surface = Gdk::Pixmap::create(Glib::RefPtr<Gdk::Drawable>(0), d.x, d.y, 24);
@@ -380,7 +389,36 @@ void OverviewMap::redraw_tiles(Rectangle tiles)
 
 	draw_terrain_tiles(Rectangle(pos, dim));
     }
-    draw(d_player);
+    draw(Playerlist::getViewingplayer());
+}
+
+Maptile* OverviewMap::getTile(int x, int y)
+{
+  //look for something interesting so we don't skip over important tiles.
+  GameMap *gm = GameMap::getInstance();
+  Maptile *favoured_tile = gm->getTile(x,y);
+  int xmax = x + map_tiles_per_tile - 1;
+  if (xmax >= GameMap::getWidth())
+    xmax = GameMap::getWidth() - 1;
+  int ymax = y + map_tiles_per_tile - 1;
+  if (ymax >= GameMap::getHeight())
+    ymax = GameMap::getHeight() - 1;
+  for (int i = x; i < xmax; i++)
+    for (int j = y; j < ymax; j++)
+      {
+        Vector<int> pos(i, j);
+        if (gm->getBuilding(pos) == Maptile::TEMPLE)
+          favoured_tile = gm->getTile(pos);
+        else if (gm->getBuilding(pos) == Maptile::RUIN)
+          favoured_tile = gm->getTile(pos);
+        else if (gm->getTerrainType(pos) == Tile::WATER)
+          favoured_tile = gm->getTile(pos);
+        else if (gm->getTerrainType(pos) == Tile::MOUNTAIN)
+          favoured_tile = gm->getTile(pos);
+        else if (gm->getTerrainType(pos) == Tile::VOID)
+          favoured_tile = gm->getTile(pos);
+      }
+  return favoured_tile;
 }
 
 void OverviewMap::draw_terrain_tiles(Rectangle r)
@@ -389,21 +427,19 @@ void OverviewMap::draw_terrain_tiles(Rectangle r)
     unsigned int oldrand = rand();
     srand(0);
     Gdk::Color rd = GameMap::getInstance()->getTileset()->getRoadColor();
-    for (int i = r.x; i < r.x + r.w; ++i)
-        for (int j = r.y; j < r.y + r.h; ++j)
+    for (int i = r.x; i < r.x + r.w; i+=int(map_tiles_per_tile))
+      for (int j = r.y; j < r.y + r.h; j+=int(map_tiles_per_tile))
         {
-            int x = int(i / pixels_per_tile);
-            int y = int(j / pixels_per_tile);
+          int x = int(i / pixels_per_tile);
+          int y = int(j / pixels_per_tile);
+          Maptile *mtile = getTile(x,y);
 
-	    if (gm->getTile(x,y)->getBuilding() == Maptile::ROAD ||
-                     gm->getTile(x,y)->getBuilding() == Maptile::BRIDGE)
-		         draw_pixel(static_surface, static_surface_gc, i, j, rd);
-	    else
-	    {
-                draw_terrain_tile (GameMap::getInstance()->getTile(x,y), i, j);
-	    }
+          if (mtile->isRoadTerrain())
+            draw_pixel(static_surface, static_surface_gc, i, j, rd);
+          else
+            draw_terrain_tile (mtile, i, j);
         }
-    srand(oldrand);
+    srand (oldrand);
 }
 
 void OverviewMap::after_draw()
@@ -412,7 +448,7 @@ void OverviewMap::after_draw()
 
 void OverviewMap::draw(Player *player)
 {
-    d_player = player;
+    Playerlist::getInstance()->setViewingplayer(player);
     int size = int(pixels_per_tile) > 1 ? int(pixels_per_tile) : 1;
     assert(surface);
 
@@ -437,9 +473,10 @@ void OverviewMap::draw(Player *player)
         it != Ruinlist::getInstance()->end(); it++)
     {
         Ruin *r = *it;
-        if (r->isHidden() == true && r->getOwner() != d_player)
+        if (r->isHidden() == true && 
+	    r->getOwner() != Playerlist::getViewingplayer())
           continue;
-        if (r->isVisible(d_player) == false)
+        if (r->isVisible(Playerlist::getViewingplayer()) == false)
           continue;
         Vector<int> pos = r->getPos();
         pos = mapToSurface(pos);
@@ -454,7 +491,7 @@ void OverviewMap::draw(Player *player)
         it != Templelist::getInstance()->end(); it++)
     {
       Temple *t = *it;
-        if (t->isVisible(d_player) == false)
+        if (t->isVisible(Playerlist::getViewingplayer()) == false)
           continue;
         Vector<int> pos = t->getPos();
         pos = mapToSurface(pos);
@@ -471,24 +508,38 @@ void OverviewMap::draw(Player *player)
           Vector <int> pos;
           pos.x = i;
           pos.y = j;
-          if (FogMap::isFogged(pos, d_player) == true)
+          if (Playerlist::getViewingplayer()->getFogMap()->isFogged(pos) == true)
             {
               pos = mapToSurface(pos);
               draw_filled_rect(true, pos.x, pos.y, size, size, fog_color);
             }
         }
 
-    // let derived classes do their job
-    after_draw();
-
-    if (d_player->getType() != Player::HUMAN &&
-	GameScenarioOptions::s_hidden_map)
+    //the idea here is that we want to show what happens when an AI-owned
+    //stack moves through our area.  so we block the view of computer
+    //players after the fact.
+    if (Playerlist::getViewingplayer()->getType() != Player::HUMAN &&
+       	GameScenarioOptions::s_hidden_map == true)
       {
 	int width = 0;
 	int height = 0;
 	surface->get_size(width, height);
-	draw_rect(true, 0, 0, width, height, fog_color);
+	draw_filled_rect(true, 0, 0, width, height, fog_color);
       }
+
+  if (blank_screen)
+    {
+      Gdk::Color fog_color = Gdk::Color();
+      fog_color.set_rgb_p(0.0,0.0,0.0);
+      int width = 0;
+      int height = 0;
+      surface->get_size(width, height);
+      surface_gc->set_rgb_fg_color(fog_color);
+      surface->draw_rectangle(surface_gc, true, 0,0,width, height);
+    }
+    // let derived classes do their job
+    after_draw();
+
 }
 
 Glib::RefPtr<Gdk::Pixmap> OverviewMap::get_surface()
@@ -498,8 +549,8 @@ Glib::RefPtr<Gdk::Pixmap> OverviewMap::get_surface()
 
 Vector<int> OverviewMap::mapFromScreen(Vector<int> pos)
 {
-    int x = int(pos.x / pixels_per_tile);
-    int y = int(pos.y / pixels_per_tile);
+    int x = int(pos.x / pixels_per_tile / map_tiles_per_tile);
+    int y = int(pos.y / pixels_per_tile / map_tiles_per_tile);
     
     if (x >= GameMap::getWidth())
         x = GameMap::getWidth() - 1;
@@ -522,8 +573,8 @@ Vector<int> OverviewMap::mapToSurface(Vector<int> pos)
     assert(pos.x >= 0 && pos.y >= 0
 	   && pos.x < GameMap::getWidth() && pos.y < GameMap::getHeight());
 
-    int x = int(round(pos.x * pixels_per_tile));
-    int y = int(round(pos.y * pixels_per_tile));
+    int x = int(round(pos.x * pixels_per_tile / map_tiles_per_tile));
+    int y = int(round(pos.y * pixels_per_tile / map_tiles_per_tile));
 
     if (pixels_per_tile > 2)
         // try to take the center position of the pixel
@@ -546,7 +597,7 @@ void OverviewMap::draw_cities (bool all_razed)
   {
       City *c = *it;
       PixMask *tmp;
-      if (c->isVisible(d_player) == false)
+      if (c->isVisible(Playerlist::getViewingplayer()) == false)
         continue;
       if (c->isBurnt() == true || all_razed == true)
         tmp = gc->getSmallRuinedCityPic();
@@ -559,3 +610,9 @@ void OverviewMap::draw_cities (bool all_razed)
 
   }
 }
+
+void OverviewMap::blank(bool on)
+{
+  blank_screen = on;
+  draw(Playerlist::getViewingplayer());
+}
