@@ -21,7 +21,6 @@
 #define RUIN_H
 
 #define DEFAULT_RUIN_NAME  "Ruin"
-#define RUIN_TILE_WIDTH 1
 #include <string>
 #include <sigc++/trackable.h>
 #include "NamedLocation.h"
@@ -30,6 +29,7 @@
 
 class Stack;
 class Reward;
+class Sage;
 
 //! A ruin on the game map.
 /** 
@@ -56,12 +56,11 @@ class Ruin : public NamedLocation, public sigc::trackable
 	  //! A stronghold ruin is a little stronger.
 	  STRONGHOLD = 1
 	};
-	static std::string ruinTypeToString(const Ruin::Type type);
-	static Ruin::Type ruinTypeFromString(const std::string str);
 
 	//! Default constructor.
         /** 
           * @param pos          The location of the ruin.
+	  * @param width        The span of tiles that the ruin covers.
           * @param name         The name of the ruin.
           * @param occupant     The monster occupying the ruin.
           * @param searched     Sets the searchedness flag of the ruin.
@@ -69,24 +68,58 @@ class Ruin : public NamedLocation, public sigc::trackable
 	  * @param owner        Who can see this hidden ruin.
 	  * @param sage         if this ruin contains a sage or not.
           */
-        Ruin(Vector<int> pos, std::string name = DEFAULT_RUIN_NAME, 
-	     int type = Ruin::RUIN, Stack* occupant = 0, 
-	     bool searched = false, bool hidden = false, Player *owner = 0, 
-	     bool sage = false);
+        Ruin(Vector<int> pos, guint32 width, 
+	     std::string name = DEFAULT_RUIN_NAME, int type = Ruin::RUIN, 
+	     Stack* occupant = 0, bool searched = false, bool hidden = false, 
+	     Player *owner = 0, bool sage = false);
 
         //! Copy constructor.
         Ruin(const Ruin&);
 
+	//! Alternative copying constructor that changes the ruin position.
+        Ruin(const Ruin&, Vector<int> pos);
+
         //! Loading constructor.
 	/**
 	 * @param helper  The opened saved-game file to load the ruin from.
 	 */
-        Ruin(XML_Helper* helper);
+        Ruin(XML_Helper* helper, guint32 tile_width);
+
 	//!Destructor.
         ~Ruin();
 
+
+	// Get Methods
+
         //! Returns the type of the ruin.
-        int getType() {return d_type;};
+        int getType() const {return d_type;};
+
+        //! Return whether or not the ruin has been searched already.
+        bool isSearched() const {return d_searched;}
+
+        //! Returns the keeper that guards the ruin from Hero units.
+        Stack* getOccupant() const {return d_occupant;}
+
+	//! Returns whether or not this is a "hidden" ruin.
+	bool isHidden() const {return d_hidden;}
+
+	//! Returns whether or not this ruin has a sage.
+	bool hasSage() const {return d_sage;}
+
+	//! Returns the player that owns this hidden ruin.
+        /**
+         * When the ruin has been searched, the owner is the player whose
+         * hero searched it.
+         */
+	Player *getOwner() const {return d_owner;}
+
+	//! Returns the reward for this ruin.
+	Reward *getReward() const {return d_reward;}
+
+	//! Returns whether or not the ruin lacks a non-default name.
+	bool isUnnamed() const {return getName() == getDefaultName() ? true : false;};
+
+	// Set Methods
 
         //! Sets the type of the ruin.
         void setType(int type) {d_type = type;};
@@ -98,41 +131,20 @@ class Ruin : public NamedLocation, public sigc::trackable
         //! Set the keeper of the ruin.
         void setOccupant(Stack* occupant) {d_occupant = occupant;}
         
-        //! Return whether or not the ruin has been searched already.
-        bool isSearched() const {return d_searched;}
-
-        //! Returns the keeper that guards the ruin from Hero units.
-        Stack* getOccupant() const {return d_occupant;}
-
-	//! Returns whether or not this is a "hidden" ruin.
-	bool isHidden() const {return d_hidden;}
-
         //! Change the "hidden" flag of the ruin.
         void setHidden (bool hidden) {d_hidden = hidden;}
 
-	//! Returns whether or not this ruin has a sage.
-	bool hasSage() const {return d_sage;}
-
 	//! Sets whether or not this ruin has a sage.
 	void setSage(bool sage) {d_sage = sage;}
 
-	//! Returns the player that owns this hidden ruin.
-	Player *getOwner() const {return d_owner;}
-
 	//! Sets the player that owns this hidden ruin.
 	void setOwner(Player *owner) {d_owner = owner;}
 
-        //! Callback for loading the ruin data.
-        bool load(std::string tag, XML_Helper* helper);
-
-	//! Returns the reward for this ruin.
-	Reward *getReward() const {return d_reward;}
-
 	//! Sets the reward for this ruin.
 	void setReward(Reward *r) {d_reward = r;}
 
-        //! Saves the ruin data to an opened saved-game file.
-        bool save(XML_Helper* helper) const;
+
+	// Methods that operate on class data and modify the class.
 
 	//! Put a random reward in this ruin.
 	/**
@@ -141,13 +153,33 @@ class Ruin : public NamedLocation, public sigc::trackable
 	 */
 	void populateWithRandomReward();
 
-	bool isUnnamed() {return getName() == getDefaultName() ? true : false;};
+
+	// Methods that operate on class data and do not modify the class.
 	
+        //! Callback for loading the ruin data.
+        bool load(std::string tag, XML_Helper* helper);
+
+        //! Saves the ruin data to an opened saved-game file.
+        bool save(XML_Helper* helper) const;
+
+        //! Create a sage object (a list of rewards).
+        Sage* generateSage() const;
+
+	// Static Methods
+
+	//! Get the default name of any ruin.
 	static std::string getDefaultName() {return _(DEFAULT_RUIN_NAME);};
-	static guint32 getWidth() {return RUIN_TILE_WIDTH;};
+
+	//! Convert a Ruin::Type enumerated value to a string.
+	static std::string ruinTypeToString(const Ruin::Type type);
+
+	//! Convert a string containing a Ruin::Type to an enumerated value.
+	static Ruin::Type ruinTypeFromString(const std::string str);
 
     private:
+
         // DATA
+
 	//! Whether or not the Ruin has already successfully been searched.
         bool d_searched;
 
