@@ -38,6 +38,7 @@
 #include "FogMap.h"
 #include "player.h"
 #include "Backpack.h"
+#include "AI_Analysis.h"
 
 std::string Stack::d_tag = "stack";
 using namespace std;
@@ -64,6 +65,10 @@ Stack::Stack(const Stack& s)
     d_defending(s.d_defending), d_parked(s.d_parked), 
     d_deleting(false)
 {
+  if (s.d_path == NULL)
+    {
+      printf("Stack %d has a null path!\n", d_id);
+    }
     d_path = new Path(*s.d_path);
 
     for (const_iterator sit = s.begin(); sit != s.end(); sit++)
@@ -126,14 +131,14 @@ bool Stack::isMovingToOrFromAShip(Vector<int> dest, bool &on_ship) const
 {
   Vector<int> pos = getPos();
 
-  bool to_city = GameMap::getInstance()->getTile(dest)->getBuilding() == Maptile::CITY;
-  bool on_city = GameMap::getInstance()->getTile(pos)->getBuilding() == Maptile::CITY;
+  bool to_city = GameMap::getInstance()->getBuilding(dest) == Maptile::CITY;
+  bool on_city = GameMap::getInstance()->getBuilding(pos) == Maptile::CITY;
 
-  bool on_port = GameMap::getInstance()->getTile(pos)->getBuilding() == Maptile::PORT;
-  bool on_bridge = GameMap::getInstance()->getTile(pos)->getBuilding() == Maptile::BRIDGE;
-  bool to_bridge = GameMap::getInstance()->getTile(dest)->getBuilding() == Maptile::BRIDGE;
-  bool on_water = (GameMap::getInstance()->getTile(pos.x,pos.y)->getMaptileType() == Tile::WATER);
-  bool to_water = (GameMap::getInstance()->getTile(dest.x,dest.y)->getMaptileType() == Tile::WATER);
+  bool on_port = GameMap::getInstance()->getBuilding(pos) == Maptile::PORT;
+  bool on_bridge = GameMap::getInstance()->getBuilding(pos) == Maptile::BRIDGE;
+  bool to_bridge = GameMap::getInstance()->getBuilding(dest) == Maptile::BRIDGE;
+  bool on_water = (GameMap::getInstance()->getTerrainType(pos) == Tile::WATER);
+  bool to_water = (GameMap::getInstance()->getTerrainType(dest) == Tile::WATER);
   //here we mark the armies as being on or off a boat
   /* skipping refers to when we have to move over another friendly stack
    * of a size that's too big to join with. */
@@ -222,20 +227,8 @@ void Stack::deFog()
   return;
 }
 
-bool Stack::isGrouped()
-{
-  if (empty())
-    return false;
-
-  for (const_iterator it = begin(); it != end(); ++it)
-    if ((*it)->isGrouped() == false)
-      return false;
-
-  return true;
-}
-
 // return the maximum moves of this stack by checking the moves of each army
-guint32 Stack::getGroupMoves() const
+guint32 Stack::getMoves() const
 {
   if (empty())
     return 0;
@@ -245,13 +238,12 @@ guint32 Stack::getGroupMoves() const
   int min = -1;
 
   for (const_iterator it = begin(); it != end(); ++it)
-    if ((*it)->isGrouped())
-      {
-	if (min == -1)
-	  min = int((*it)->getMoves());
-	else
-	  min = std::min(min, int((*it)->getMoves()));
-      }
+    {
+      if (min == -1)
+	min = int((*it)->getMoves());
+      else
+	min = std::min(min, int((*it)->getMoves()));
+    }
 
   if (min <= -1)
     return 0;
@@ -344,41 +336,6 @@ Army *Stack::getArmyById(guint32 id) const
   return 0;
 }
 
-guint32 Stack::countGroupedArmies() const
-{
-  guint32 count = 0;
-  if (empty())
-    return 0;
-  for (const_iterator it = begin(); it != end(); ++it)
-    if ((*it)->isGrouped() == true)
-      count++;
-  return count;
-}
-
-void Stack::group()
-{
-  debug("Stack::group()")
-  if (empty())
-    return;
-  for (iterator it = begin(); it != end(); ++it)
-    (*it)->setGrouped(true);
-  sgrouped.emit(this, true);
-  return;
-}
-
-void Stack::ungroup()
-{
-  debug("Stack::ungroup()")
-  if (empty())
-    return;
-  for (iterator it = begin(); it != end(); ++it)
-    (*it)->setGrouped(false);
-  //set first army to be in the group
-  (*(begin()))->setGrouped(true);
-  sgrouped.emit(this, false);
-  return;
-}
-
 bool Stack::hasHero() const
 {
   for (const_iterator it = begin(); it != end(); it++)
@@ -414,7 +371,7 @@ int Stack::bless()
   int count = 0;
   for (iterator it = begin(); it != end(); it++)
     {
-      Temple *temple = GameMap::getTemple(getPos());
+      Temple *temple = GameMap::getTemple(this);
       if ((*it)->bless(temple))
 	count++;
     }
@@ -435,6 +392,8 @@ guint32 Stack::calculateTileMovementCost(Vector<int> pos) const
 
 Vector<int> Stack::getFirstPointInPath() const
 {
+  if (d_path->size() == 0)
+    return Vector<int>(-1,-1);
   Vector<int> p = *(d_path->begin());
   return p;
 }
@@ -468,7 +427,7 @@ bool Stack::enoughMoves() const
   Vector<int> p = getFirstPointInPath();
   guint32 needed = calculateTileMovementCost(p);
 
-  if (getGroupMoves() >= needed)
+  if (getMoves() >= needed)
     return true;
 
   return false;
@@ -477,24 +436,12 @@ bool Stack::enoughMoves() const
 bool Stack::canMove() const
 {
   int tile_moves = getMinTileMoves();
-  int group_moves = getGroupMoves();
+  int group_moves = getMoves();
 
   assert (tile_moves != -1);
   return group_moves > 0 && tile_moves >= 0 && group_moves >= tile_moves;
 }
 
-Army* Stack::getFirstUngroupedArmy() const
-{
-  for (const_iterator it = begin(); it != end(); it++)
-    {
-      if (!(*it)->isGrouped())
-	{
-	  return *it;
-	}
-    }
-  return 0;
-}
-
 guint32 Stack::getMaxSight() const
 {
   guint32 max = 0;
@@ -628,11 +575,8 @@ guint32 Stack::calculateMoveBonus() const
   int num_landedother = 0;
   if (size() == 0)
     return 0;
-  bool grouped = countGroupedArmies() > 0;
   for (const_iterator it = this->begin(); it != this->end(); it++)
     {
-      if ((*it)->isGrouped() == false && grouped)
-	continue;
       bonus = (*it)->getStat(Army::MOVE_BONUS);
       if (bonus == Tile::GRASS || (bonus & Tile::WATER) == 0 || 
 	  (bonus & Tile::FOREST) == 0 || (bonus & Tile::HILLS) == 0 ||
@@ -673,8 +617,6 @@ guint32 Stack::calculateMoveBonus() const
   //calculate move bonuses for non-flying stacks
   for (Stack::const_iterator it = this->begin(); it != this->end(); it++)
     {
-      if ((*it)->isGrouped() == false)
-	continue;
       bonus = (*it)->getStat(Army::MOVE_BONUS);
 
       //only forest and hills extend to all other units in the stack
@@ -699,8 +641,6 @@ bool Stack::hasShip () const
 {
   for (Stack::const_iterator it = this->begin(); it != this->end(); it++)
     {
-      if ((*it)->isGrouped() == false)
-	continue;
       if ((*it)->getStat(Army::SHIP))
 	return true;
     }
@@ -720,6 +660,11 @@ guint32 getFightOrder(std::list<guint32> values, guint32 value)
   return 0;
 }
 
+bool Stack::armyCompareStrength (const Army *lhs, const Army *rhs)  
+{
+  return lhs->getStat(Army::STRENGTH) < rhs->getStat(Army::STRENGTH);
+}
+
 bool Stack::armyCompareFightOrder (const Army *lhs, const Army *rhs)  
 {
   std::list<guint32> lhs_fight_order = lhs->getOwner()->getFightOrder();
@@ -729,16 +674,16 @@ bool Stack::armyCompareFightOrder (const Army *lhs, const Army *rhs)
   return lhs_rank < rhs_rank; 
 }
 
-bool armyCompareGrouped (const Army *lhs, const Army *rhs)  
+void Stack::sortByStrength(bool reverse)
 {
-  return lhs->isGrouped() < rhs->isGrouped(); 
+  sort(armyCompareStrength);
+  if (reverse)
+    std::reverse(begin(), end());
 }
 
-
 void Stack::sortForViewing (bool reverse)
 {
   sort(armyCompareFightOrder);
-  sort(armyCompareGrouped);
   if (reverse)
     std::reverse(begin(), end());
 }
@@ -753,11 +698,11 @@ void Stack::setFortified(bool fortified)
   (*begin())->setFortified(fortified);
 }
 
-bool Stack::getFortified()
+bool Stack::getFortified() const
 {
   if (empty())
     return false;
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getFortified())
 	return true;
@@ -765,84 +710,112 @@ bool Stack::getFortified()
   return false;
 }
 
-guint32 Stack::getUpkeep()
+guint32 Stack::getUpkeep() const
 {
   guint32 upkeep = 0;
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     upkeep += (*it)->getUpkeep();
   return upkeep;
 }
 
+guint32 Stack::getMaxArmiesToJoin() const
+{
+  return MAX_STACK_SIZE - size();
+}
+
 bool Stack::canJoin(const Stack *stack) const
 {
-  guint32 joinSize = countGroupedArmies();
-  if (joinSize == 0)
-    joinSize = size();
-
-  /* this is necessary because human players "group" armies in stacks,
-   * while the computer player does not.
-   */
-  if ((stack->size() + joinSize) > MAX_STACK_SIZE)
+  if ((stack->size() + size()) > MAX_STACK_SIZE)
     return false;
 
   return true;
 
 }
 
-std::vector<guint32> Stack::determineReachableArmies(Vector<int> dest)
+//take the weakest units where their strengths add up to strength.
+std::list<guint32> Stack::determineArmiesByStrength(bool strongest, float strength) const
 {
-  std::vector<guint32> ids;
+  std::list<guint32> armies;
+  float remaining = strength; 
+  Stack *stack = new Stack(*this);
+  stack->sortByStrength(false);
+  for (iterator it = stack->begin(); it != stack->end(); it++)
+    {
+      float score = AI_Analysis::assessArmyStrength(*it);
+      if (score > remaining)
+        continue;
+      else
+        {
+          remaining -= score;
+          armies.push_back((*it)->getId());
+        }
+    }
+  delete stack;
+  return armies;
+}
+
+std::list<guint32> Stack::determineStrongArmies(float strength) const
+{
+  return determineArmiesByStrength(true, strength);
+}
+std::list<guint32> Stack::determineWeakArmies(float strength) const
+{
+  return determineArmiesByStrength(false, strength);
+}
+
+std::list<guint32> Stack::determineReachableArmies(Vector<int> dest) const
+{
+  std::list<guint32> ids;
   //try each army individually to see if it reaches
-  for (iterator it = begin(); it != end(); it++)
+	  
+  Stack *stack = Stack::createNonUniqueStack(getOwner(), getPos());
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if ((*it)->getMoves() > 0)
 	{
-	  Stack *stack = Stack::createNonUniqueStack(getOwner(), getPos());
 	  stack->push_back(*it);
-	  if (stack->getGroupMoves() >= 
-	      stack->getPath()->calculate(stack, dest))
+	  if (stack->getMoves() >= stack->getPath()->calculate(stack, dest))
 	    ids.push_back((*it)->getId());
 	  stack->clear();
-	  delete stack;
-	  //fixme: can't delete stacks like this because it deletes the army
 	}
     }
+  delete stack;
   if (ids.size() == 0)
     return ids;
 
   //now try to see if any army units can tag along
-  for (iterator it = begin(); it != end(); it++)
+  stack = Stack::createNonUniqueStack(getOwner(), getPos());
+  for (const_iterator it = begin(); it != end(); it++)
     {
       //skip over armies that are already known to be reachable
       if (find(ids.begin(), ids.end(), (*it)->getId()) != ids.end())
 	continue;
       if ((*it)->getMoves() > 0)
 	{
-	  Stack *stack = Stack::createNonUniqueStack(getOwner(), getPos());
 	  stack->push_back(*it);
 	  //also push back the rest of the known reachables
-	  std::vector<guint32>::iterator iit = ids.begin();
+	  std::list<guint32>::iterator iit = ids.begin();
 	  for (; iit != ids.end(); iit++)
 	    {
 	      Army *army = getArmyById(*iit);
 	      if (army)
 		stack->push_back(army);
 	    }
-	  if (stack->getGroupMoves() >= 
+	  if (stack->getMoves() >= 
 	      stack->getPath()->calculate(stack, dest))
 	    ids.push_back((*it)->getId());
 	  stack->clear();
-	  delete stack;
 	}
     }
+  delete stack;
 
   return ids;
 }
 
-guint32 Stack::countArmiesBlessedAtTemple(guint32 temple_id)
+guint32 Stack::countArmiesBlessedAtTemple(guint32 temple_id) const
 {
   guint32 blessed = 0;
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = begin(); it != end(); it++)
     {
       if ((*it)->blessedAtTemple(temple_id))
 	blessed++;
@@ -855,7 +828,7 @@ Stack* Stack::createNonUniqueStack(Player *player, Vector<int> pos)
   return new Stack(0, player, pos);
 }
 
-guint32 Stack::getMaxGroupLandMoves() const
+guint32 Stack::getMaxLandMoves() const
 {
   if (empty())
     return 0;
@@ -865,11 +838,9 @@ guint32 Stack::getMaxGroupLandMoves() const
   //copy the stack, reset the moves and return the group moves
   Stack *copy = new Stack (*this);
   copy->getPath()->clear(); //this prevents triggering path recalc in nextTurn
-  if (copy->countGroupedArmies() == 0)
-    copy->group();
-  copy->decrementMoves(copy->getGroupMoves());
+  copy->decrementMoves(copy->getMoves());
   copy->nextTurn();
-  guint32 moves = copy->getGroupMoves();
+  guint32 moves = copy->getMoves();
   if (isFlying() == true)
     {
       delete copy;
@@ -878,17 +849,17 @@ guint32 Stack::getMaxGroupLandMoves() const
 
   //alright, we're not flying.  what would our group moves be if we were on land
   //remove ship status from all army units
-  copy->decrementMoves(copy->getGroupMoves());
+  copy->decrementMoves(copy->getMoves());
   for (Stack::iterator it = copy->begin(); it != copy->end(); it++)
     (*it)->setInShip(false);
   copy->nextTurn();
 
-  moves = copy->getGroupMoves();
+  moves = copy->getMoves();
   delete copy;
   return moves;
 }
 
-guint32 Stack::getMaxGroupBoatMoves() const
+guint32 Stack::getMaxBoatMoves() const
 {
   if (empty())
     return 0;
@@ -898,17 +869,15 @@ guint32 Stack::getMaxGroupBoatMoves() const
   //copy the stack, reset the moves and return the group moves
   Stack *copy = new Stack (*this);
   copy->getPath()->clear(); //this prevents triggering path recalc in nextTurn
-  if (copy->countGroupedArmies() == 0)
-    copy->group();
   copy->nextTurn();
-  guint32 moves = copy->getGroupMoves();
+  guint32 moves = copy->getMoves();
   if (isFlying() == true)
     {
       delete copy;
       return moves;
     }
   //alright, we're not flying.  what would our group moves be if we were on water?
-  copy->decrementMoves(copy->getGroupMoves());
+  copy->decrementMoves(copy->getMoves());
 	      
   for (Stack::iterator it = copy->begin(); it != copy->end(); it++)
     {
@@ -919,7 +888,7 @@ guint32 Stack::getMaxGroupBoatMoves() const
     }
   copy->nextTurn();
 
-  moves = copy->getGroupMoves();
+  moves = copy->getMoves();
   delete copy;
   return moves;
 }
@@ -930,4 +899,198 @@ void Stack::setPath(const Path p)
     delete d_path;
   d_path = new Path(p);
 }
+
+void Stack::add(Army *army)
+{
+  push_back(army);
+}
+
+//! split the given army from this stack, into a brand new stack.
+Stack *Stack::splitArmy(Army *army)
+{
+  if (size() == 1) //we can't split the last army.
+    return NULL;
+
+  assert (army != NULL);
+  Stack *new_stack = NULL;
+  for (iterator it = begin(); it != end(); it++)
+    {
+      if (*it == army || (*it)->getId() == army->getId())
+	{
+	  new_stack = new Stack(getOwner(), getPos());
+	  new_stack->add(*it);
+	  it = erase(it);
+	  break;
+	}
+    }
+
+  return new_stack;
+}
+
+//! split the given armies from this stack, into a brand new stack.
+Stack *Stack::splitArmies(std::list<Army*> armies)
+{
+  std::list<guint32> ids;
+  for (std::list<Army*>::iterator i = armies.begin(); i != armies.end(); i++)
+    ids.push_back((*i)->getId());
+  return splitArmies(ids);
+}
+
+Stack *Stack::splitArmies(std::list<guint32> armies)
+{
+  if (armies.size() == 0) //we can't split 0 armies into a new stack.
+    return NULL;
+  if (armies.size() >= size()) //we can't split everyone into a new stack.
+    return NULL;
+  Stack *new_stack = NULL;
+  for (std::list<guint32>::iterator i = armies.begin(); i != armies.end(); i++)
+    {
+      bool found = false;
+      iterator found_army_it = end();
+      for (iterator it = begin(); it != end(); it++)
+	{
+	  if ((*it)->getId() == *i)
+	    {
+	      found = true;
+	      found_army_it = it;
+	      break;
+	    }
+	}
+      if (found)
+	{
+	  if (new_stack == NULL)
+	    new_stack = new Stack(getOwner(), getPos());
+	  new_stack->push_back(*found_army_it);
+	  erase(found_army_it);
+	}
+    }
+  return new_stack;
+}
+//! split the armies in the stack that this much mp or more into a new stack.
+Stack *Stack::splitArmiesWithMovement(guint32 mp)
+{
+  std::list<Army*> armies;
+  for (iterator it = begin(); it != end(); it++)
+    if ((*it)->getMoves() >= mp)
+      armies.push_back(*it);
+  return splitArmies(armies);
+}
+
+void Stack::join(Stack *join)
+{
+  for (iterator i = join->begin(); i != join->end(); i++)
+    push_back(*i);
+  join->clear();
+}
+
+bool Stack::validate() const
+{
+  if (size() > MAX_STACK_SIZE)
+    return false;
+  if (size() == 0)
+    return false;
+  return true;
+}
+
+bool Stack::isFull() const
+{
+  if (size() >= MAX_STACK_SIZE)
+    return true;
+  return false;
+}
+
+bool Stack::clearPath()
+{
+  if (getPath())
+    {
+      if (getPath()->size() > 0)
+	{
+	  getPath()->clear();
+	  return true;
+	}
+      else
+	return false;
+    }
+  else
+    return false;
+  return true;
+}
+
+bool Stack::isOnCity() const
+{
+  if (GameMap::getInstance()->getBuilding(getPos()) == Maptile::CITY)
+    return true;
+  return false;
+}
+
+bool Stack::hasPath() const
+{
+  if (getPath() && getPath()->size() > 0)
+    return true;
+  return false;
+}
+
+bool Stack::hasQuest() const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->isHero() == true)
+        {
+          Hero *hero = dynamic_cast<Hero*>(*it);
+          if (hero->hasQuest() == true)
+            return true;
+
+        }
+    }
+  return false;
+}
+
+bool Stack::hasArmyType(guint32 army_type) const
+{
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->getTypeId() == army_type)
+        return true;
+    }
+  return false;
+}
+
+guint32 Stack::countAllies() const
+{
+  guint32 count = 0;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->getAwardable() == true)
+        count++;
+    }
+  return count;
+}
+
+Hero *Stack::getFirstHeroWithoutAQuest() const
+{
+  Hero *hero = NULL;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->isHero() == false)
+        continue;
+      hero = dynamic_cast<Hero*>(*it);
+      if (hero->hasQuest() == false)
+        return hero;
+    }
+  return NULL;
+}
+
+Hero *Stack::getFirstHeroWithAQuest() const
+{
+  Hero *hero = NULL;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if ((*it)->isHero() == false)
+        continue;
+      hero = dynamic_cast<Hero*>(*it);
+      if (hero->hasQuest() == true)
+        return hero;
+    }
+  return NULL;
+}
 // End of file
