@@ -2,7 +2,7 @@
 // Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Ulf Lorenz
 // Copyright (C) 2004, 2005 Andrea Paternesi
 // Copyright (C) 2004 John Farrell
-// Copyright (C) 2007, 2008, 2009 Ben Asselstine
+// Copyright (C) 2007, 2008, 2009, 2010 Ben Asselstine
 // Copyright (C) 2007, 2008 Ole Laursen
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -37,6 +37,8 @@
 #include "hero.h"
 #include "Backpack.h"
 #include "LocationList.h"
+#include "GameMap.h"
+#include "stacktile.h"
 
 std::string Stacklist::d_tag = "stacklist";
 using namespace std;
@@ -44,24 +46,6 @@ using namespace std;
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 #define debug(x)
 
-//the static functions first
-Stack* Stacklist::getObjectAt(int x, int y)
-{
-    for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
-        pit != Playerlist::getInstance()->end(); pit++)
-    {
-        Stacklist* mylist = (*pit)->getStacklist();
-	Stack *s1 = NULL;
-	Stack *s2 = NULL;
-	mylist->getObjectAt(Vector<int>(x,y), s1, s2);
-	if (s1)
-	  return s1;
-	else if (s2)
-	  return s2;
-    }
-    return 0;
-}
-
 Vector<int> Stacklist::getPosition(guint32 id)
 {
     for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
@@ -83,20 +67,7 @@ Vector<int> Stacklist::getPosition(guint32 id)
 //It also happens when two stacks fight.
 Stack* Stacklist::getAmbiguity(Stack* s)
 {
-    for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
-        pit != Playerlist::getInstance()->end(); pit++)
-    {
-        Stacklist* mylist = (*pit)->getStacklist();
-	Stack *s1 = NULL;
-	Stack *s2 = NULL;
-	mylist->getObjectAt(s->getPos(), s1, s2);
-	if (s1 && s1->getId() != s->getId())
-	  return s1;
-	else if (s2 && s2->getId() != s->getId())
-	  return s2;
-    }
-
-    return 0;
+  return GameMap::getInstance()->getTile(s->getPos())->getStacks()->getOtherStack(s);
 }
 
 //search all player's stacklists to find this stack
@@ -131,6 +102,37 @@ void Stacklist::payUpkeep(Player *p)
     (*it)->payUpkeep(p);
 }
 
+bool Stacklist::check()
+{
+  //printf("checking stacks of the current player.  the current player is %s\n", Playerlist::getActiveplayer()->getName().c_str());
+  //if (size())
+    //printf("stacklist stacks belong to %s\n", front()->getOwner()->getName().c_str());
+    for (iterator it = begin(); it != end(); it++)
+      {
+        if ((*it)->getOwner()->isComputer() == false)
+          continue;
+        std::list<Stack*> f = GameMap::getFriendlyStacks((*it)->getPos());
+        if (f.size() > 1)
+          {
+	      fprintf (stderr, "%d stacks found on %d,%d\n", f.size(),
+                       (*it)->getPos().x, (*it)->getPos().y);
+              for (std::list<Stack*>::iterator t = f.begin(); t != f.end(); t++)
+                {
+                  Stack *stack = *t;
+                  if (stack)
+                    {
+                    printf("stack id: %d\n", stack->getId());
+                    printf("\tsize is %d\n", stack->size());
+                    }
+                  else
+                    printf("null stack\n");
+                }
+              return false;
+          }
+      }
+    return true;
+}
+
 void Stacklist::nextTurn()
 {
     debug("nextTurn()");
@@ -146,11 +148,13 @@ void Stacklist::nextTurn()
 	      fprintf (stderr, "duplicate army id %d found\n", (*it)->getId());
 	      exit (1);
 	    }
+    //printf("checking at next-turn time\n");
+    //check();
 }
 
-vector<Stack*> Stacklist::defendersInCity(City *city)
+vector<Stack*> Stacklist::getDefendersInCity(const City *city)
 {
-    debug("defendersInCity()");
+    debug("getDefendersInCity()");
 
     vector<Stack*> stackvector;
     Vector<int> pos = city->getPos();
@@ -159,12 +163,12 @@ vector<Stack*> Stacklist::defendersInCity(City *city)
     {
         for (unsigned int j = pos.y; j < pos.y + city->getSize(); j++)
         {
-	    Stack *stack;
-	    stack = city->getOwner()->getStacklist()->getOwnObjectAt(i, j);
-            if (stack)
-            {
-	      stackvector.push_back(stack);
-            }
+	    Vector<int> p = Vector<int>(i,j);
+	    std::list<Stack *>stacks = 
+	      GameMap::getFriendlyStacks(p, city->getOwner());
+	    for (std::list<Stack*>::iterator it = stacks.begin(); 
+		 it != stacks.end(); it++)
+		stackvector.push_back(*it);
         }
     }
 
@@ -184,21 +188,39 @@ unsigned int Stacklist::getNoOfStacks()
     return mysize;
 }
 
-unsigned int Stacklist::countArmies()
+unsigned int Stacklist::getNoOfArmies()
 {
     unsigned int mysize = 0;
 
-    for (iterator it = begin(); it != end(); it++)
+    for (Playerlist::iterator pit = Playerlist::getInstance()->begin();
+        pit != Playerlist::getInstance()->end(); pit++)
+    {
+        mysize += (*pit)->getStacklist()->countArmies();
+    }
+
+    return mysize;
+}
+
+unsigned int Stacklist::countArmies() const
+{
+    unsigned int mysize = 0;
+
+    for (const_iterator it = begin(); it != end(); it++)
       mysize += (*it)->size();
 
     return mysize;
 }
 
-unsigned int Stacklist::countHeroes()
+unsigned int Stacklist::countAllies() const
 {
-  std::vector<guint32> heroes;
-  getHeroes(heroes);
-  return heroes.size();
+    unsigned int mysize = 0;
+
+    for (const_iterator it = begin(); it != end(); it++)
+      {
+        mysize += (*it)->countAllies();
+      }
+
+    return mysize;
 }
 
 Stacklist::Stacklist()
@@ -240,11 +262,11 @@ Stacklist::~Stacklist()
 
 }
 
-Stack* Stacklist::getNextMovable()
+Stack* Stacklist::getNextMovable() const
 {
     Player *player = Playerlist::getInstance()->getActiveplayer();
     
-    iterator it = begin();
+    const_iterator it = begin();
     
     //first, if we already have an active stack, loop through until we meet it
     if (d_activestack)
@@ -283,18 +305,18 @@ Stack* Stacklist::getNextMovable()
 	return d_activestack;
 }
 
-Stack *Stacklist::getStackById(guint32 id)
+Stack *Stacklist::getStackById(guint32 id) const
 {
-  for (Stacklist::iterator i = begin(), e = end(); i != e; ++i)
-    if ((*i)->getId() == id)
-      return *i;
-    
-  return 0;
+  IdMap::const_iterator it = d_id.find(id);
+  if (it != d_id.end())
+    return (*it).second;
+  else
+    return NULL;
 }
 
-Stack *Stacklist::getArmyStackById(guint32 army)
+Stack *Stacklist::getArmyStackById(guint32 army) const
 {
-  for (Stacklist::iterator i = begin(), e = end(); i != e; ++i)
+  for (Stacklist::const_iterator i = begin(), e = end(); i != e; ++i)
     if ((*i)->getArmyById(army))
       return *i;
   
@@ -393,10 +415,8 @@ bool Stacklist::load(string tag, XML_Helper* helper)
     if (tag == Stack::d_tag)
     {
         Stack* s = new Stack(helper);
-        if ((active != 0) && (s->getId() == active))
-        {
-            d_activestack = s;
-        }
+        if (active != 0 && s->getId() == active)
+	  d_activestack = s;
 
         add(s);
         return true;
@@ -405,21 +425,13 @@ bool Stacklist::load(string tag, XML_Helper* helper)
     return false;
 }
 
-Stack* Stacklist::getOwnObjectAt(int x, int y)
+void Stacklist::getHeroes(std::vector<guint32>& dst) const
 {
-  for (const_iterator it = begin(); it != end(); it++)
-    if (((*it)->getPos().x == x) && ((*it)->getPos().y == y))
-      return *it;
-  return 0;
-}
-
-void Stacklist::getHeroes(std::vector<guint32>& dst)
-{
-  for (Stacklist::iterator it = begin(); it != end(); it++)
+  for (Stacklist::const_iterator it = begin(); it != end(); it++)
     (*it)->getHeroes(dst);
 }
 
-void Stacklist::collectTaxes(Player *p, guint32 num_cities)
+void Stacklist::collectTaxes(Player *p, guint32 num_cities) const
 {
   std::vector<guint32> hero_ids;
   getHeroes(hero_ids);
@@ -446,39 +458,29 @@ void Stacklist::collectTaxes(Player *p, guint32 num_cities)
 bool Stacklist::canJumpOverTooLargeStack(Stack *s)
 {
   bool found = false;
-  guint32 mp = s->getGroupMoves();
+  guint32 mp = s->getMoves();
   for (Path::iterator it = s->getPath()->begin(); it != s->getPath()->end(); it++)
     {
       guint32 moves = s->calculateTileMovementCost(*it);
       if (moves > mp)
 	return false;
       mp -= moves;
-      Stack *another_stack = getObjectAt(*it);
-      if (another_stack)
-	{
-	  if (another_stack->getOwner() != s->getOwner())
-	    return false;
-	  if (s->canJoin(another_stack) == true)
-	    {
-	      found = true;
-	      break;
-	    }
-	}
-      else
-	{
-	  found = true;
-	  break;
-	}
+      if (GameMap::getEnemyCity(*it) != NULL)
+        return false;
+      if (GameMap::getEnemyStack(*it) != NULL)
+        return false;
+      if (GameMap::canJoin(s, *it) == true)
+	return true;
     }
   return found;
 }
 
-std::list<Hero*> Stacklist::getHeroes()
+std::list<Hero*> Stacklist::getHeroes() const
 {
   std::list<Hero*> heroes;
   std::vector<guint32> hero_ids;
   getHeroes(hero_ids);
-  for (std::vector<guint32>::iterator it = hero_ids.begin(); 
+  for (std::vector<guint32>::const_iterator it = hero_ids.begin(); 
        it != hero_ids.end(); it++)
     {
         Stack *s = getArmyStackById(*it);
@@ -492,7 +494,7 @@ std::list<Hero*> Stacklist::getHeroes()
   return heroes;
 }
 	
-Hero *Stacklist::getNearestHero(Vector<int> pos, int dist)
+Hero *Stacklist::getNearestHero(Vector<int> pos, int dist) const
 {
   std::list<Hero*> heroes = getHeroes();
   LocationList<Location*> hero_locales;
@@ -515,54 +517,20 @@ Hero *Stacklist::getNearestHero(Vector<int> pos, int dist)
 
 bool Stacklist::addPositionToMap(Stack *stack)
 {
-  if (d_object1.find(stack->getPos()) == d_object1.end())
-    {
-      d_object1[stack->getPos()] = stack;
-      return true;
-    }
-
-  if (d_object2.find(stack->getPos()) == d_object2.end())
-    {
-      d_object2[stack->getPos()] = stack;
-      return true;
-    }
-  
-  return false;
+  snewpos.emit(stack, stack->getPos());
+  return true;
 }
 
 bool Stacklist::deletePositionFromMap(Stack *stack)
 {
-  Stack *found = NULL;
-  PositionMap::iterator it = d_object1.find(stack->getPos());
-  if (it != d_object1.end())
-    {
-      found = (*it).second;
-      if (found->getId() == stack->getId())
-	{
-	  d_object1.erase(it);
-	  //d_object1[stack->getPos()] = NULL;
-	  return true;
-	}
-    }
-
-  it = d_object2.find(stack->getPos());
-  if (it != d_object2.end())
-    {
-      found = (*it).second;
-      if (found->getId() == stack->getId())
-	{
-	  //d_object2[stack->getPos()] = NULL;
-	  d_object2.erase(it);
-	  return true;
-	}
-    }
-  
-  return false;
+  soldpos.emit(stack, stack->getPos());
+  return true;
 }
 
 void Stacklist::add(Stack *stack)
 {
   push_back(stack);
+  d_id[stack->getId()] = stack;
   if (stack->getPos() != Vector<int>(-1,-1))
     {
       bool added = addPositionToMap(stack);
@@ -595,6 +563,7 @@ void Stacklist::on_stack_died (Stack *stack)
       for (std::list<sigc::connection>::iterator lit = list.begin(); lit != list.end(); lit++)
 	(*lit).disconnect();
     }
+  d_id.erase(d_id.find(stack->getId()));
   return;
 }
 void Stacklist::on_stack_starts_moving (Stack *stack)
@@ -607,20 +576,6 @@ void Stacklist::on_stack_stops_moving (Stack *stack)
   addPositionToMap(stack);
   return;
 }
-void Stacklist::getObjectAt(Vector<int> pos, Stack *& s1, Stack *&s2)
-{
-  PositionMap::iterator it = d_object1.find(pos);
-  if (it == d_object1.end())
-    s1 = NULL;
-  else
-    s1 = (*it).second;
-  it = d_object2.find(pos);
-  if (it == d_object2.end())
-    s2 = NULL;
-  else
-    s2 = (*it).second;
-}
-        
 void Stacklist::setActivestack(Stack* activestack)
 {
   d_activestack = activestack;
@@ -632,4 +587,27 @@ void Stacklist::drainAllMovement()
     for (Stack::iterator ait = (*it)->begin(); ait != (*it)->end(); ait++)
       (*ait)->decrementMoves((*ait)->getMoves());
 }
+
+void Stacklist::changeOwnership(Stack *stack, Player *new_owner)
+{
+  if (new_owner != stack->getOwner())
+    {
+      Stack *new_stack = new Stack(*stack);
+      stack->getOwner()->getStacklist()->flRemove(stack);
+      new_owner->addStack(new_stack);
+    }
+}
+
+std::list<Vector<int> > Stacklist::getPositions() const
+{
+  std::list<Vector<int> > points;
+  for (const_iterator it = begin(); it != end(); it++)
+    {
+      if (std::find(points.begin(), points.end(), (*it)->getPos()) == 
+          points.end())
+        points.push_back((*it)->getPos());
+    }
+  return points;
+
+}
 // End of file
